"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComplexityHandler = exports.ComplexityMathDocumentMixin = exports.ComplexityMathItemMixin = void 0;
const MathItem_js_1 = require("../core/MathItem.js");
const semantic_enrich_js_1 = require("./semantic-enrich.js");
const visitor_js_1 = require("./complexity/visitor.js");
const Options_js_1 = require("../util/Options.js");
(0, MathItem_js_1.newState)('COMPLEXITY', 40);
function ComplexityMathItemMixin(BaseMathItem, computeComplexity) {
    return class extends BaseMathItem {
        complexity(document, force = false) {
            if (this.state() >= MathItem_js_1.STATE.COMPLEXITY)
                return;
            if (!this.isEscaped && (document.options.enableComplexity || force)) {
                this.enrich(document, true);
                computeComplexity(this.root);
            }
            this.state(MathItem_js_1.STATE.COMPLEXITY);
        }
    };
}
exports.ComplexityMathItemMixin = ComplexityMathItemMixin;
function ComplexityMathDocumentMixin(BaseDocument) {
    var _a;
    return _a = class extends BaseDocument {
            constructor(...args) {
                super(...args);
                const ProcessBits = this.constructor.ProcessBits;
                if (!ProcessBits.has('complexity')) {
                    ProcessBits.allocate('complexity');
                }
                const visitorOptions = (0, Options_js_1.selectOptionsFromKeys)(this.options, this.options.ComplexityVisitor.OPTIONS);
                this.complexityVisitor = new this.options.ComplexityVisitor(this.mmlFactory, visitorOptions);
                const computeComplexity = ((node) => this.complexityVisitor.visitTree(node));
                this.options.MathItem =
                    ComplexityMathItemMixin(this.options.MathItem, computeComplexity);
            }
            complexity() {
                if (!this.processed.isSet('complexity')) {
                    if (this.options.enableComplexity) {
                        for (const math of this.math) {
                            math.complexity(this);
                        }
                    }
                    this.processed.set('complexity');
                }
                return this;
            }
            state(state, restore = false) {
                super.state(state, restore);
                if (state < MathItem_js_1.STATE.COMPLEXITY) {
                    this.processed.clear('complexity');
                }
                return this;
            }
        },
        _a.OPTIONS = Object.assign(Object.assign(Object.assign({}, BaseDocument.OPTIONS), visitor_js_1.ComplexityVisitor.OPTIONS), { enableComplexity: true, ComplexityVisitor: visitor_js_1.ComplexityVisitor, renderActions: (0, Options_js_1.expandable)(Object.assign(Object.assign({}, BaseDocument.OPTIONS.renderActions), { complexity: [MathItem_js_1.STATE.COMPLEXITY] })) }),
        _a;
}
exports.ComplexityMathDocumentMixin = ComplexityMathDocumentMixin;
function ComplexityHandler(handler, MmlJax = null) {
    if (!handler.documentClass.prototype.enrich && MmlJax) {
        handler = (0, semantic_enrich_js_1.EnrichHandler)(handler, MmlJax);
    }
    handler.documentClass = ComplexityMathDocumentMixin(handler.documentClass);
    return handler;
}
exports.ComplexityHandler = ComplexityHandler;
//# sourceMappingURL=complexity.js.map