"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractExplorer = void 0;
const sre_js_1 = __importDefault(require("../sre.js"));
class AbstractExplorer {
    static stopEvent(event) {
        if (event.preventDefault) {
            event.preventDefault();
        }
        else {
            event.returnValue = false;
        }
        if (event.stopImmediatePropagation) {
            event.stopImmediatePropagation();
        }
        else if (event.stopPropagation) {
            event.stopPropagation();
        }
        event.cancelBubble = true;
    }
    static create(document, region, node, ...rest) {
        let explorer = new this(document, region, node, ...rest);
        return explorer;
    }
    constructor(document, region, node, ..._rest) {
        this.document = document;
        this.region = region;
        this.node = node;
        this.stoppable = true;
        this.events = [];
        this.highlighter = this.getHighlighter();
        this._active = false;
    }
    Events() {
        return this.events;
    }
    get active() {
        return this._active;
    }
    set active(flag) {
        this._active = flag;
    }
    Attach() {
        this.AddEvents();
    }
    Detach() {
        this.RemoveEvents();
    }
    Start() {
        this.highlighter = this.getHighlighter();
        this.active = true;
    }
    Stop() {
        if (this.active) {
            this.region.Clear();
            this.region.Hide();
            this.active = false;
        }
    }
    AddEvents() {
        for (let [eventkind, eventfunc] of this.events) {
            this.node.addEventListener(eventkind, eventfunc);
        }
    }
    RemoveEvents() {
        for (let [eventkind, eventfunc] of this.events) {
            this.node.removeEventListener(eventkind, eventfunc);
        }
    }
    Update(force = false) { }
    getHighlighter() {
        let opts = this.document.options.a11y;
        let foreground = { color: opts.foregroundColor.toLowerCase(),
            alpha: opts.foregroundOpacity / 100 };
        let background = { color: opts.backgroundColor.toLowerCase(),
            alpha: opts.backgroundOpacity / 100 };
        return sre_js_1.default.getHighlighter(background, foreground, { renderer: this.document.outputJax.name, browser: 'v3' });
    }
    stopEvent(event) {
        if (this.stoppable) {
            AbstractExplorer.stopEvent(event);
        }
    }
}
exports.AbstractExplorer = AbstractExplorer;
//# sourceMappingURL=Explorer.js.map