"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FlameHoverer = exports.ContentHoverer = exports.ValueHoverer = exports.Hoverer = exports.AbstractMouseExplorer = void 0;
const Region_js_1 = require("./Region.js");
const Explorer_js_1 = require("./Explorer.js");
require("../sre.js");
class AbstractMouseExplorer extends Explorer_js_1.AbstractExplorer {
    constructor() {
        super(...arguments);
        this.events = super.Events().concat([
            ['mouseover', this.MouseOver.bind(this)],
            ['mouseout', this.MouseOut.bind(this)]
        ]);
    }
    MouseOver(_event) {
        this.Start();
    }
    MouseOut(_event) {
        this.Stop();
    }
}
exports.AbstractMouseExplorer = AbstractMouseExplorer;
class Hoverer extends AbstractMouseExplorer {
    constructor(document, region, node, nodeQuery, nodeAccess) {
        super(document, region, node);
        this.document = document;
        this.region = region;
        this.node = node;
        this.nodeQuery = nodeQuery;
        this.nodeAccess = nodeAccess;
    }
    MouseOut(event) {
        if (event.clientX === this.coord[0] &&
            event.clientY === this.coord[1]) {
            return;
        }
        this.highlighter.unhighlight();
        this.region.Hide();
        super.MouseOut(event);
    }
    MouseOver(event) {
        super.MouseOver(event);
        let target = event.target;
        this.coord = [event.clientX, event.clientY];
        let [node, kind] = this.getNode(target);
        if (!node) {
            return;
        }
        this.highlighter.unhighlight();
        this.highlighter.highlight([node]);
        this.region.Update(kind);
        this.region.Show(node, this.highlighter);
    }
    getNode(node) {
        let original = node;
        while (node && node !== this.node) {
            if (this.nodeQuery(node)) {
                return [node, this.nodeAccess(node)];
            }
            node = node.parentNode;
        }
        node = original;
        while (node) {
            if (this.nodeQuery(node)) {
                return [node, this.nodeAccess(node)];
            }
            let child = node.childNodes[0];
            node = (child && child.tagName === 'defs') ?
                node.childNodes[1] : child;
        }
        return [null, null];
    }
}
exports.Hoverer = Hoverer;
class ValueHoverer extends Hoverer {
}
exports.ValueHoverer = ValueHoverer;
class ContentHoverer extends Hoverer {
}
exports.ContentHoverer = ContentHoverer;
class FlameHoverer extends Hoverer {
    constructor(document, _ignore, node) {
        super(document, new Region_js_1.DummyRegion(document), node, x => this.highlighter.isMactionNode(x), () => { });
        this.document = document;
        this.node = node;
    }
}
exports.FlameHoverer = FlameHoverer;
//# sourceMappingURL=MouseExplorer.js.map