"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnrichHandler = exports.EnrichedMathDocumentMixin = exports.EnrichedMathItemMixin = void 0;
const mathjax_js_1 = require("../mathjax.js");
const MathItem_js_1 = require("../core/MathItem.js");
const SerializedMmlVisitor_js_1 = require("../core/MmlTree/SerializedMmlVisitor.js");
const Options_js_1 = require("../util/Options.js");
const sre_js_1 = __importDefault(require("./sre.js"));
let currentSpeech = 'none';
(0, MathItem_js_1.newState)('ENRICHED', 30);
(0, MathItem_js_1.newState)('ATTACHSPEECH', 155);
function EnrichedMathItemMixin(BaseMathItem, MmlJax, toMathML) {
    return class extends BaseMathItem {
        serializeMml(node) {
            if ('outerHTML' in node) {
                return node.outerHTML;
            }
            if (typeof Element !== 'undefined' && typeof window !== 'undefined' && node instanceof Element) {
                const div = window.document.createElement('div');
                div.appendChild(node);
                return div.innerHTML;
            }
            return node.toString();
        }
        enrich(document, force = false) {
            if (this.state() >= MathItem_js_1.STATE.ENRICHED)
                return;
            if (!this.isEscaped && (document.options.enableEnrichment || force)) {
                if (document.options.sre.speech !== currentSpeech) {
                    currentSpeech = document.options.sre.speech;
                    mathjax_js_1.mathjax.retryAfter(sre_js_1.default.setupEngine(document.options.sre).then(() => sre_js_1.default.sreReady()));
                }
                const math = new document.options.MathItem('', MmlJax);
                try {
                    const mml = this.inputData.originalMml = toMathML(this.root);
                    math.math = this.serializeMml(sre_js_1.default.toEnriched(mml));
                    math.display = this.display;
                    math.compile(document);
                    this.root = math.root;
                    this.inputData.enrichedMml = math.math;
                }
                catch (err) {
                    document.options.enrichError(document, this, err);
                }
            }
            this.state(MathItem_js_1.STATE.ENRICHED);
        }
        attachSpeech(document) {
            if (this.state() >= MathItem_js_1.STATE.ATTACHSPEECH)
                return;
            const attributes = this.root.attributes;
            const speech = (attributes.get('aria-label') ||
                this.getSpeech(this.root));
            if (speech) {
                const adaptor = document.adaptor;
                const node = this.typesetRoot;
                adaptor.setAttribute(node, 'aria-label', speech);
                for (const child of adaptor.childNodes(node)) {
                    adaptor.setAttribute(child, 'aria-hidden', 'true');
                }
            }
            this.state(MathItem_js_1.STATE.ATTACHSPEECH);
        }
        getSpeech(node) {
            const attributes = node.attributes;
            if (!attributes)
                return '';
            const speech = attributes.getExplicit('data-semantic-speech');
            if (!attributes.getExplicit('data-semantic-parent') && speech) {
                return speech;
            }
            for (let child of node.childNodes) {
                let value = this.getSpeech(child);
                if (value != null) {
                    return value;
                }
            }
            return '';
        }
    };
}
exports.EnrichedMathItemMixin = EnrichedMathItemMixin;
function EnrichedMathDocumentMixin(BaseDocument, MmlJax) {
    var _a;
    return _a = class extends BaseDocument {
            constructor(...args) {
                super(...args);
                MmlJax.setMmlFactory(this.mmlFactory);
                const ProcessBits = this.constructor.ProcessBits;
                if (!ProcessBits.has('enriched')) {
                    ProcessBits.allocate('enriched');
                    ProcessBits.allocate('attach-speech');
                }
                const visitor = new SerializedMmlVisitor_js_1.SerializedMmlVisitor(this.mmlFactory);
                const toMathML = ((node) => visitor.visitTree(node));
                this.options.MathItem =
                    EnrichedMathItemMixin(this.options.MathItem, MmlJax, toMathML);
            }
            attachSpeech() {
                if (!this.processed.isSet('attach-speech')) {
                    for (const math of this.math) {
                        math.attachSpeech(this);
                    }
                    this.processed.set('attach-speech');
                }
                return this;
            }
            enrich() {
                if (!this.processed.isSet('enriched')) {
                    if (this.options.enableEnrichment) {
                        for (const math of this.math) {
                            math.enrich(this);
                        }
                    }
                    this.processed.set('enriched');
                }
                return this;
            }
            enrichError(_doc, _math, err) {
                console.warn('Enrichment error:', err);
            }
            state(state, restore = false) {
                super.state(state, restore);
                if (state < MathItem_js_1.STATE.ENRICHED) {
                    this.processed.clear('enriched');
                }
                return this;
            }
        },
        _a.OPTIONS = Object.assign(Object.assign({}, BaseDocument.OPTIONS), { enableEnrichment: true, enrichError: (doc, math, err) => doc.enrichError(doc, math, err), renderActions: (0, Options_js_1.expandable)(Object.assign(Object.assign({}, BaseDocument.OPTIONS.renderActions), { enrich: [MathItem_js_1.STATE.ENRICHED], attachSpeech: [MathItem_js_1.STATE.ATTACHSPEECH] })), sre: (0, Options_js_1.expandable)({
                speech: 'none',
                domain: 'mathspeak',
                style: 'default',
                locale: 'en'
            }) }),
        _a;
}
exports.EnrichedMathDocumentMixin = EnrichedMathDocumentMixin;
function EnrichHandler(handler, MmlJax) {
    MmlJax.setAdaptor(handler.adaptor);
    handler.documentClass =
        EnrichedMathDocumentMixin(handler.documentClass, MmlJax);
    return handler;
}
exports.EnrichHandler = EnrichHandler;
//# sourceMappingURL=semantic-enrich.js.map