"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDOMAdaptor = void 0;
class AbstractDOMAdaptor {
    constructor(document = null) {
        this.document = document;
    }
    node(kind, def = {}, children = [], ns) {
        const node = this.create(kind, ns);
        this.setAttributes(node, def);
        for (const child of children) {
            this.append(node, child);
        }
        return node;
    }
    setAttributes(node, def) {
        if (def.style && typeof (def.style) !== 'string') {
            for (let key of Object.keys(def.style)) {
                this.setStyle(node, key.replace(/-([a-z])/g, (_m, c) => c.toUpperCase()), def.style[key]);
            }
        }
        if (def.properties) {
            for (let key of Object.keys(def.properties)) {
                node[key] = def.properties[key];
            }
        }
        for (let key of Object.keys(def)) {
            if ((key !== 'style' || typeof (def.style) === 'string') && key !== 'properties') {
                this.setAttribute(node, key, def[key]);
            }
        }
    }
    replace(nnode, onode) {
        this.insert(nnode, onode);
        this.remove(onode);
        return onode;
    }
    childNode(node, i) {
        return this.childNodes(node)[i];
    }
    allClasses(node) {
        const classes = this.getAttribute(node, 'class');
        return (!classes ? [] :
            classes.replace(/  +/g, ' ').replace(/^ /, '').replace(/ $/, '').split(/ /));
    }
}
exports.AbstractDOMAdaptor = AbstractDOMAdaptor;
//# sourceMappingURL=DOMAdaptor.js.map