"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonMmlVisitor = void 0;
const MmlVisitor_js_1 = require("./MmlVisitor.js");
class JsonMmlVisitor extends MmlVisitor_js_1.MmlVisitor {
    visitTree(node) {
        return this.visitNode(node);
    }
    visitTextNode(node) {
        return { kind: node.kind, text: node.getText() };
    }
    visitXMLNode(node) {
        return { kind: node.kind, xml: node.getXML() };
    }
    visitDefault(node) {
        let json = {
            kind: node.kind.replace(/inferredM/, 'm'),
            texClass: node.texClass,
            attributes: this.getAttributes(node),
            inherited: this.getInherited(node),
            properties: this.getProperties(node),
            childNodes: this.getChildren(node)
        };
        if (node.isInferred) {
            json.isInferred = true;
        }
        if (node.isEmbellished) {
            json.isEmbellished = true;
        }
        if (node.isSpacelike) {
            json.isSpacelike = true;
        }
        return json;
    }
    getChildren(node) {
        let children = [];
        for (const child of node.childNodes) {
            children.push(this.visitNode(child));
        }
        return children;
    }
    getAttributes(node) {
        return Object.assign({}, node.attributes.getAllAttributes());
    }
    getInherited(node) {
        return Object.assign({}, node.attributes.getAllInherited());
    }
    getProperties(node) {
        return Object.assign({}, node.getAllProperties());
    }
}
exports.JsonMmlVisitor = JsonMmlVisitor;
//# sourceMappingURL=JsonMmlVisitor.js.map