"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MathMLVisitor = void 0;
const MmlVisitor_js_1 = require("./MmlVisitor.js");
class MathMLVisitor extends MmlVisitor_js_1.MmlVisitor {
    constructor() {
        super(...arguments);
        this.document = null;
    }
    visitTree(node, document) {
        this.document = document;
        let root = document.createElement('top');
        this.visitNode(node, root);
        this.document = null;
        return root.firstChild;
    }
    visitTextNode(node, parent) {
        parent.appendChild(this.document.createTextNode(node.getText()));
    }
    visitXMLNode(node, parent) {
        parent.appendChild(node.getXML().cloneNode(true));
    }
    visitInferredMrowNode(node, parent) {
        for (const child of node.childNodes) {
            this.visitNode(child, parent);
        }
    }
    visitDefault(node, parent) {
        let mml = this.document.createElement(node.kind);
        this.addAttributes(node, mml);
        for (const child of node.childNodes) {
            this.visitNode(child, mml);
        }
        parent.appendChild(mml);
    }
    addAttributes(node, mml) {
        let attributes = node.attributes;
        let names = attributes.getExplicitNames();
        for (const name of names) {
            mml.setAttribute(name, attributes.getExplicit(name).toString());
        }
    }
}
exports.MathMLVisitor = MathMLVisitor;
//# sourceMappingURL=MathMLVisitor.js.map