"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MmlNone = exports.MmlMprescripts = exports.MmlMmultiscripts = void 0;
const MmlNode_js_1 = require("../MmlNode.js");
const msubsup_js_1 = require("./msubsup.js");
class MmlMmultiscripts extends msubsup_js_1.MmlMsubsup {
    get kind() {
        return 'mmultiscripts';
    }
    get arity() {
        return 1;
    }
    setChildInheritedAttributes(attributes, display, level, prime) {
        this.childNodes[0].setInheritedAttributes(attributes, display, level, prime);
        let prescripts = false;
        for (let i = 1, n = 0; i < this.childNodes.length; i++) {
            let child = this.childNodes[i];
            if (child.isKind('mprescripts')) {
                if (!prescripts) {
                    prescripts = true;
                    if (i % 2 === 0) {
                        let mrow = this.factory.create('mrow');
                        this.childNodes.splice(i, 0, mrow);
                        mrow.parent = this;
                        i++;
                    }
                }
            }
            else {
                let primestyle = prime || (n % 2 === 0);
                child.setInheritedAttributes(attributes, false, level + 1, primestyle);
                n++;
            }
        }
        if (this.childNodes.length % 2 === (prescripts ? 1 : 0)) {
            this.appendChild(this.factory.create('mrow'));
            this.childNodes[this.childNodes.length - 1].setInheritedAttributes(attributes, false, level + 1, prime);
        }
    }
    verifyChildren(options) {
        let prescripts = false;
        let fix = options['fixMmultiscripts'];
        for (let i = 0; i < this.childNodes.length; i++) {
            let child = this.childNodes[i];
            if (child.isKind('mprescripts')) {
                if (prescripts) {
                    child.mError(child.kind + ' can only appear once in ' + this.kind, options, true);
                }
                else {
                    prescripts = true;
                    if (i % 2 === 0 && !fix) {
                        this.mError('There must be an equal number of prescripts of each type', options);
                    }
                }
            }
        }
        if (this.childNodes.length % 2 === (prescripts ? 1 : 0) && !fix) {
            this.mError('There must be an equal number of scripts of each type', options);
        }
        super.verifyChildren(options);
    }
}
exports.MmlMmultiscripts = MmlMmultiscripts;
MmlMmultiscripts.defaults = Object.assign({}, msubsup_js_1.MmlMsubsup.defaults);
class MmlMprescripts extends MmlNode_js_1.AbstractMmlNode {
    get kind() {
        return 'mprescripts';
    }
    get arity() {
        return 0;
    }
    verifyTree(options) {
        super.verifyTree(options);
        if (this.parent && !this.parent.isKind('mmultiscripts')) {
            this.mError(this.kind + ' must be a child of mmultiscripts', options, true);
        }
    }
}
exports.MmlMprescripts = MmlMprescripts;
MmlMprescripts.defaults = Object.assign({}, MmlNode_js_1.AbstractMmlNode.defaults);
class MmlNone extends MmlNode_js_1.AbstractMmlNode {
    get kind() {
        return 'none';
    }
    get arity() {
        return 0;
    }
    verifyTree(options) {
        super.verifyTree(options);
        if (this.parent && !this.parent.isKind('mmultiscripts')) {
            this.mError(this.kind + ' must be a child of mmultiscripts', options, true);
        }
    }
}
exports.MmlNone = MmlNone;
MmlNone.defaults = Object.assign({}, MmlNode_js_1.AbstractMmlNode.defaults);
//# sourceMappingURL=mmultiscripts.js.map