"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MmlMo = void 0;
const MmlNode_js_1 = require("../MmlNode.js");
const OperatorDictionary_js_1 = require("../OperatorDictionary.js");
const string_js_1 = require("../../../util/string.js");
class MmlMo extends MmlNode_js_1.AbstractMmlTokenNode {
    constructor() {
        super(...arguments);
        this._texClass = null;
        this.lspace = 5 / 18;
        this.rspace = 5 / 18;
    }
    get texClass() {
        if (this._texClass === null) {
            let mo = this.getText();
            let [form1, form2, form3] = this.handleExplicitForm(this.getForms());
            let OPTABLE = this.constructor.OPTABLE;
            let def = OPTABLE[form1][mo] || OPTABLE[form2][mo] || OPTABLE[form3][mo];
            return def ? def[2] : MmlNode_js_1.TEXCLASS.REL;
        }
        return this._texClass;
    }
    set texClass(value) {
        this._texClass = value;
    }
    get kind() {
        return 'mo';
    }
    get isEmbellished() {
        return true;
    }
    get hasNewLine() {
        return this.attributes.get('linebreak') === 'newline';
    }
    coreParent() {
        let embellished = this;
        let parent = this;
        let math = this.factory.getNodeClass('math');
        while (parent && parent.isEmbellished && parent.coreMO() === this && !(parent instanceof math)) {
            embellished = parent;
            parent = parent.parent;
        }
        return embellished;
    }
    coreText(parent) {
        if (!parent) {
            return '';
        }
        if (parent.isEmbellished) {
            return parent.coreMO().getText();
        }
        while ((((parent.isKind('mrow') ||
            (parent.isKind('TeXAtom') && parent.texClass !== MmlNode_js_1.TEXCLASS.VCENTER) ||
            parent.isKind('mstyle') ||
            parent.isKind('mphantom')) && parent.childNodes.length === 1) ||
            parent.isKind('munderover')) && parent.childNodes[0]) {
            parent = parent.childNodes[0];
        }
        return (parent.isToken ? parent.getText() : '');
    }
    hasSpacingAttributes() {
        return this.attributes.isSet('lspace') ||
            this.attributes.isSet('rspace');
    }
    get isAccent() {
        let accent = false;
        const node = this.coreParent().parent;
        if (node) {
            const key = (node.isKind('mover') ?
                (node.childNodes[node.over].coreMO() ?
                    'accent' : '') :
                node.isKind('munder') ?
                    (node.childNodes[node.under].coreMO() ?
                        'accentunder' : '') :
                    node.isKind('munderover') ?
                        (this === node.childNodes[node.over].coreMO() ?
                            'accent' :
                            this === node.childNodes[node.under].coreMO() ?
                                'accentunder' : '') :
                        '');
            if (key) {
                const value = node.attributes.getExplicit(key);
                accent = (value !== undefined ? accent : this.attributes.get('accent'));
            }
        }
        return accent;
    }
    setTeXclass(prev) {
        let { form, fence } = this.attributes.getList('form', 'fence');
        if (this.getProperty('texClass') === undefined &&
            (this.attributes.isSet('lspace') || this.attributes.isSet('rspace'))) {
            return null;
        }
        if (fence && this.texClass === MmlNode_js_1.TEXCLASS.REL) {
            if (form === 'prefix') {
                this.texClass = MmlNode_js_1.TEXCLASS.OPEN;
            }
            if (form === 'postfix') {
                this.texClass = MmlNode_js_1.TEXCLASS.CLOSE;
            }
        }
        return this.adjustTeXclass(prev);
    }
    adjustTeXclass(prev) {
        let texClass = this.texClass;
        let prevClass = this.prevClass;
        if (texClass === MmlNode_js_1.TEXCLASS.NONE) {
            return prev;
        }
        if (prev) {
            if (prev.getProperty('autoOP') && (texClass === MmlNode_js_1.TEXCLASS.BIN || texClass === MmlNode_js_1.TEXCLASS.REL)) {
                prevClass = prev.texClass = MmlNode_js_1.TEXCLASS.ORD;
            }
            prevClass = this.prevClass = (prev.texClass || MmlNode_js_1.TEXCLASS.ORD);
            this.prevLevel = this.attributes.getInherited('scriptlevel');
        }
        else {
            prevClass = this.prevClass = MmlNode_js_1.TEXCLASS.NONE;
        }
        if (texClass === MmlNode_js_1.TEXCLASS.BIN &&
            (prevClass === MmlNode_js_1.TEXCLASS.NONE || prevClass === MmlNode_js_1.TEXCLASS.BIN || prevClass === MmlNode_js_1.TEXCLASS.OP ||
                prevClass === MmlNode_js_1.TEXCLASS.REL || prevClass === MmlNode_js_1.TEXCLASS.OPEN || prevClass === MmlNode_js_1.TEXCLASS.PUNCT)) {
            this.texClass = MmlNode_js_1.TEXCLASS.ORD;
        }
        else if (prevClass === MmlNode_js_1.TEXCLASS.BIN &&
            (texClass === MmlNode_js_1.TEXCLASS.REL || texClass === MmlNode_js_1.TEXCLASS.CLOSE || texClass === MmlNode_js_1.TEXCLASS.PUNCT)) {
            prev.texClass = this.prevClass = MmlNode_js_1.TEXCLASS.ORD;
        }
        else if (texClass === MmlNode_js_1.TEXCLASS.BIN) {
            let child = this;
            let parent = this.parent;
            while (parent && parent.parent && parent.isEmbellished &&
                (parent.childNodes.length === 1 ||
                    (!parent.isKind('mrow') && parent.core() === child))) {
                child = parent;
                parent = parent.parent;
            }
            if (parent.childNodes[parent.childNodes.length - 1] === child) {
                this.texClass = MmlNode_js_1.TEXCLASS.ORD;
            }
        }
        return this;
    }
    setInheritedAttributes(attributes = {}, display = false, level = 0, prime = false) {
        super.setInheritedAttributes(attributes, display, level, prime);
        let mo = this.getText();
        this.checkOperatorTable(mo);
        this.checkPseudoScripts(mo);
        this.checkPrimes(mo);
        this.checkMathAccent(mo);
    }
    checkOperatorTable(mo) {
        let [form1, form2, form3] = this.handleExplicitForm(this.getForms());
        this.attributes.setInherited('form', form1);
        let OPTABLE = this.constructor.OPTABLE;
        let def = OPTABLE[form1][mo] || OPTABLE[form2][mo] || OPTABLE[form3][mo];
        if (def) {
            if (this.getProperty('texClass') === undefined) {
                this.texClass = def[2];
            }
            for (const name of Object.keys(def[3] || {})) {
                this.attributes.setInherited(name, def[3][name]);
            }
            this.lspace = (def[0] + 1) / 18;
            this.rspace = (def[1] + 1) / 18;
        }
        else {
            let range = (0, OperatorDictionary_js_1.getRange)(mo);
            if (range) {
                if (this.getProperty('texClass') === undefined) {
                    this.texClass = range[2];
                }
                const spacing = this.constructor.MMLSPACING[range[2]];
                this.lspace = (spacing[0] + 1) / 18;
                this.rspace = (spacing[1] + 1) / 18;
            }
        }
    }
    getForms() {
        let core = this;
        let parent = this.parent;
        let Parent = this.Parent;
        while (Parent && Parent.isEmbellished) {
            core = parent;
            parent = Parent.parent;
            Parent = Parent.Parent;
        }
        if (parent && parent.isKind('mrow') && parent.nonSpaceLength() !== 1) {
            if (parent.firstNonSpace() === core) {
                return ['prefix', 'infix', 'postfix'];
            }
            if (parent.lastNonSpace() === core) {
                return ['postfix', 'infix', 'prefix'];
            }
        }
        return ['infix', 'prefix', 'postfix'];
    }
    handleExplicitForm(forms) {
        if (this.attributes.isSet('form')) {
            const form = this.attributes.get('form');
            forms = [form].concat(forms.filter(name => (name !== form)));
        }
        return forms;
    }
    checkPseudoScripts(mo) {
        const PSEUDOSCRIPTS = this.constructor.pseudoScripts;
        if (!mo.match(PSEUDOSCRIPTS))
            return;
        const parent = this.coreParent().Parent;
        const isPseudo = !parent || !(parent.isKind('msubsup') && !parent.isKind('msub'));
        this.setProperty('pseudoscript', isPseudo);
        if (isPseudo) {
            this.attributes.setInherited('lspace', 0);
            this.attributes.setInherited('rspace', 0);
        }
    }
    checkPrimes(mo) {
        const PRIMES = this.constructor.primes;
        if (!mo.match(PRIMES))
            return;
        const REMAP = this.constructor.remapPrimes;
        const primes = (0, string_js_1.unicodeString)((0, string_js_1.unicodeChars)(mo).map(c => REMAP[c]));
        this.setProperty('primes', primes);
    }
    checkMathAccent(mo) {
        const parent = this.Parent;
        if (this.getProperty('mathaccent') !== undefined || !parent || !parent.isKind('munderover'))
            return;
        const base = parent.childNodes[0];
        if (base.isEmbellished && base.coreMO() === this)
            return;
        const MATHACCENT = this.constructor.mathaccents;
        if (mo.match(MATHACCENT)) {
            this.setProperty('mathaccent', true);
        }
    }
}
exports.MmlMo = MmlMo;
MmlMo.defaults = Object.assign(Object.assign({}, MmlNode_js_1.AbstractMmlTokenNode.defaults), { form: 'infix', fence: false, separator: false, lspace: 'thickmathspace', rspace: 'thickmathspace', stretchy: false, symmetric: false, maxsize: 'infinity', minsize: '0em', largeop: false, movablelimits: false, accent: false, linebreak: 'auto', lineleading: '1ex', linebreakstyle: 'before', indentalign: 'auto', indentshift: '0', indenttarget: '', indentalignfirst: 'indentalign', indentshiftfirst: 'indentshift', indentalignlast: 'indentalign', indentshiftlast: 'indentshift' });
MmlMo.MMLSPACING = OperatorDictionary_js_1.MMLSPACING;
MmlMo.OPTABLE = OperatorDictionary_js_1.OPTABLE;
MmlMo.pseudoScripts = new RegExp([
    '^["\'*`',
    '\u00AA',
    '\u00B0',
    '\u00B2-\u00B4',
    '\u00B9',
    '\u00BA',
    '\u2018-\u201F',
    '\u2032-\u2037\u2057',
    '\u2070\u2071',
    '\u2074-\u207F',
    '\u2080-\u208E',
    ']+$'
].join(''));
MmlMo.primes = new RegExp([
    '^["\'`',
    '\u2018-\u201F',
    ']+$'
].join(''));
MmlMo.remapPrimes = {
    0x0022: 0x2033,
    0x0027: 0x2032,
    0x0060: 0x2035,
    0x2018: 0x2035,
    0x2019: 0x2032,
    0x201A: 0x2032,
    0x201B: 0x2035,
    0x201C: 0x2036,
    0x201D: 0x2033,
    0x201E: 0x2033,
    0x201F: 0x2036,
};
MmlMo.mathaccents = new RegExp([
    '^[',
    '\u00B4\u0301\u02CA',
    '\u0060\u0300\u02CB',
    '\u00A8\u0308',
    '\u007E\u0303\u02DC',
    '\u00AF\u0304\u02C9',
    '\u02D8\u0306',
    '\u02C7\u030C',
    '\u005E\u0302\u02C6',
    '\u2192\u20D7',
    '\u02D9\u0307',
    '\u02DA\u030A',
    '\u20DB',
    '\u20DC',
    ']$'
].join(''));
//# sourceMappingURL=mo.js.map