"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFactory = void 0;
class AbstractFactory {
    constructor(nodes = null) {
        this.defaultKind = 'unknown';
        this.nodeMap = new Map();
        this.node = {};
        if (nodes === null) {
            nodes = this.constructor.defaultNodes;
        }
        for (const kind of Object.keys(nodes)) {
            this.setNodeClass(kind, nodes[kind]);
        }
    }
    create(kind, ...args) {
        return (this.node[kind] || this.node[this.defaultKind])(...args);
    }
    setNodeClass(kind, nodeClass) {
        this.nodeMap.set(kind, nodeClass);
        let THIS = this;
        let KIND = this.nodeMap.get(kind);
        this.node[kind] = (...args) => {
            return new KIND(THIS, ...args);
        };
    }
    getNodeClass(kind) {
        return this.nodeMap.get(kind);
    }
    deleteNodeClass(kind) {
        this.nodeMap.delete(kind);
        delete this.node[kind];
    }
    nodeIsKind(node, kind) {
        return (node instanceof this.getNodeClass(kind));
    }
    getKinds() {
        return Array.from(this.nodeMap.keys());
    }
}
exports.AbstractFactory = AbstractFactory;
AbstractFactory.defaultNodes = {};
//# sourceMappingURL=Factory.js.map