"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MathML = void 0;
const InputJax_js_1 = require("../core/InputJax.js");
const Options_js_1 = require("../util/Options.js");
const FunctionList_js_1 = require("../util/FunctionList.js");
const FindMathML_js_1 = require("./mathml/FindMathML.js");
const MathMLCompile_js_1 = require("./mathml/MathMLCompile.js");
class MathML extends InputJax_js_1.AbstractInputJax {
    constructor(options = {}) {
        let [mml, find, compile] = (0, Options_js_1.separateOptions)(options, FindMathML_js_1.FindMathML.OPTIONS, MathMLCompile_js_1.MathMLCompile.OPTIONS);
        super(mml);
        this.findMathML = this.options['FindMathML'] || new FindMathML_js_1.FindMathML(find);
        this.mathml = this.options['MathMLCompile'] || new MathMLCompile_js_1.MathMLCompile(compile);
        this.mmlFilters = new FunctionList_js_1.FunctionList();
    }
    setAdaptor(adaptor) {
        super.setAdaptor(adaptor);
        this.findMathML.adaptor = adaptor;
        this.mathml.adaptor = adaptor;
    }
    setMmlFactory(mmlFactory) {
        super.setMmlFactory(mmlFactory);
        this.mathml.setMmlFactory(mmlFactory);
    }
    get processStrings() {
        return false;
    }
    compile(math, document) {
        let mml = math.start.node;
        if (!mml || !math.end.node || this.options['forceReparse'] || this.adaptor.kind(mml) === '#text') {
            let mathml = this.executeFilters(this.preFilters, math, document, (math.math || '<math></math>').trim());
            let doc = this.checkForErrors(this.adaptor.parse(mathml, 'text/' + this.options['parseAs']));
            let body = this.adaptor.body(doc);
            if (this.adaptor.childNodes(body).length !== 1) {
                this.error('MathML must consist of a single element');
            }
            mml = this.adaptor.remove(this.adaptor.firstChild(body));
            if (this.adaptor.kind(mml).replace(/^[a-z]+:/, '') !== 'math') {
                this.error('MathML must be formed by a <math> element, not <' + this.adaptor.kind(mml) + '>');
            }
        }
        mml = this.executeFilters(this.mmlFilters, math, document, mml);
        return this.executeFilters(this.postFilters, math, document, this.mathml.compile(mml));
    }
    checkForErrors(doc) {
        let err = this.adaptor.tags(this.adaptor.body(doc), 'parsererror')[0];
        if (err) {
            if (this.adaptor.textContent(err) === '') {
                this.error('Error processing MathML');
            }
            this.options['parseError'].call(this, err);
        }
        return doc;
    }
    error(message) {
        throw new Error(message);
    }
    findMath(node) {
        return this.findMathML.findMath(node);
    }
}
exports.MathML = MathML;
MathML.NAME = 'MathML';
MathML.OPTIONS = (0, Options_js_1.defaultOptions)({
    parseAs: 'html',
    forceReparse: false,
    FindMathML: null,
    MathMLCompile: null,
    parseError: function (node) {
        this.error(this.adaptor.textContent(node).replace(/\n.*/g, ''));
    }
}, InputJax_js_1.AbstractInputJax.OPTIONS);
//# sourceMappingURL=mathml.js.map