"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FindMathML = void 0;
const FindMath_js_1 = require("../../core/FindMath.js");
const NAMESPACE = 'http://www.w3.org/1998/Math/MathML';
class FindMathML extends FindMath_js_1.AbstractFindMath {
    findMath(node) {
        let set = new Set();
        this.findMathNodes(node, set);
        this.findMathPrefixed(node, set);
        const html = this.adaptor.root(this.adaptor.document);
        if (this.adaptor.kind(html) === 'html' && set.size === 0) {
            this.findMathNS(node, set);
        }
        return this.processMath(set);
    }
    findMathNodes(node, set) {
        for (const math of this.adaptor.tags(node, 'math')) {
            set.add(math);
        }
    }
    findMathPrefixed(node, set) {
        let html = this.adaptor.root(this.adaptor.document);
        for (const attr of this.adaptor.allAttributes(html)) {
            if (attr.name.substr(0, 6) === 'xmlns:' && attr.value === NAMESPACE) {
                let prefix = attr.name.substr(6);
                for (const math of this.adaptor.tags(node, prefix + ':math')) {
                    set.add(math);
                }
            }
        }
    }
    findMathNS(node, set) {
        for (const math of this.adaptor.tags(node, 'math', NAMESPACE)) {
            set.add(math);
        }
    }
    processMath(set) {
        let math = [];
        for (const mml of Array.from(set)) {
            let display = (this.adaptor.getAttribute(mml, 'display') === 'block' ||
                this.adaptor.getAttribute(mml, 'mode') === 'display');
            let start = { node: mml, n: 0, delim: '' };
            let end = { node: mml, n: 0, delim: '' };
            math.push({ math: this.adaptor.outerHTML(mml), start, end, display });
        }
        return math;
    }
}
exports.FindMathML = FindMathML;
FindMathML.OPTIONS = {};
//# sourceMappingURL=FindMathML.js.map