"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MathMLCompile = void 0;
const MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
const Options_js_1 = require("../../util/Options.js");
const Entities = __importStar(require("../../util/Entities.js"));
class MathMLCompile {
    constructor(options = {}) {
        const Class = this.constructor;
        this.options = (0, Options_js_1.userOptions)((0, Options_js_1.defaultOptions)({}, Class.OPTIONS), options);
    }
    setMmlFactory(mmlFactory) {
        this.factory = mmlFactory;
    }
    compile(node) {
        let mml = this.makeNode(node);
        mml.verifyTree(this.options['verify']);
        mml.setInheritedAttributes({}, false, 0, false);
        mml.walkTree(this.markMrows);
        return mml;
    }
    makeNode(node) {
        const adaptor = this.adaptor;
        let limits = false;
        let kind = adaptor.kind(node).replace(/^.*:/, '');
        let texClass = adaptor.getAttribute(node, 'data-mjx-texclass') || '';
        if (texClass) {
            texClass = this.filterAttribute('data-mjx-texclass', texClass) || '';
        }
        let type = texClass && kind === 'mrow' ? 'TeXAtom' : kind;
        for (const name of this.filterClassList(adaptor.allClasses(node))) {
            if (name.match(/^MJX-TeXAtom-/) && kind === 'mrow') {
                texClass = name.substr(12);
                type = 'TeXAtom';
            }
            else if (name === 'MJX-fixedlimits') {
                limits = true;
            }
        }
        this.factory.getNodeClass(type) || this.error('Unknown node type "' + type + '"');
        let mml = this.factory.create(type);
        if (type === 'TeXAtom' && texClass === 'OP' && !limits) {
            mml.setProperty('movesupsub', true);
            mml.attributes.setInherited('movablelimits', true);
        }
        if (texClass) {
            mml.texClass = MmlNode_js_1.TEXCLASS[texClass];
            mml.setProperty('texClass', mml.texClass);
        }
        this.addAttributes(mml, node);
        this.checkClass(mml, node);
        this.addChildren(mml, node);
        return mml;
    }
    addAttributes(mml, node) {
        let ignoreVariant = false;
        for (const attr of this.adaptor.allAttributes(node)) {
            let name = attr.name;
            let value = this.filterAttribute(name, attr.value);
            if (value === null || name === 'xmlns') {
                continue;
            }
            if (name.substr(0, 9) === 'data-mjx-') {
                switch (name.substr(9)) {
                    case 'alternate':
                        mml.setProperty('variantForm', true);
                        break;
                    case 'variant':
                        mml.attributes.set('mathvariant', value);
                        ignoreVariant = true;
                        break;
                    case 'smallmatrix':
                        mml.setProperty('scriptlevel', 1);
                        mml.setProperty('useHeight', false);
                        break;
                    case 'accent':
                        mml.setProperty('mathaccent', value === 'true');
                        break;
                    case 'auto-op':
                        mml.setProperty('autoOP', value === 'true');
                        break;
                    case 'script-align':
                        mml.setProperty('scriptalign', value);
                        break;
                }
            }
            else if (name !== 'class') {
                let val = value.toLowerCase();
                if (val === 'true' || val === 'false') {
                    mml.attributes.set(name, val === 'true');
                }
                else if (!ignoreVariant || name !== 'mathvariant') {
                    mml.attributes.set(name, value);
                }
            }
        }
    }
    filterAttribute(_name, value) {
        return value;
    }
    filterClassList(list) {
        return list;
    }
    addChildren(mml, node) {
        if (mml.arity === 0) {
            return;
        }
        const adaptor = this.adaptor;
        for (const child of adaptor.childNodes(node)) {
            const name = adaptor.kind(child);
            if (name === '#comment') {
                continue;
            }
            if (name === '#text') {
                this.addText(mml, child);
            }
            else if (mml.isKind('annotation-xml')) {
                mml.appendChild(this.factory.create('XML').setXML(child, adaptor));
            }
            else {
                let childMml = mml.appendChild(this.makeNode(child));
                if (childMml.arity === 0 && adaptor.childNodes(child).length) {
                    if (this.options['fixMisplacedChildren']) {
                        this.addChildren(mml, child);
                    }
                    else {
                        childMml.mError('There should not be children for ' + childMml.kind + ' nodes', this.options['verify'], true);
                    }
                }
            }
        }
    }
    addText(mml, child) {
        let text = this.adaptor.value(child);
        if ((mml.isToken || mml.getProperty('isChars')) && mml.arity) {
            if (mml.isToken) {
                text = Entities.translate(text);
                text = this.trimSpace(text);
            }
            mml.appendChild(this.factory.create('text').setText(text));
        }
        else if (text.match(/\S/)) {
            this.error('Unexpected text node "' + text + '"');
        }
    }
    checkClass(mml, node) {
        let classList = [];
        for (const name of this.filterClassList(this.adaptor.allClasses(node))) {
            if (name.substr(0, 4) === 'MJX-') {
                if (name === 'MJX-variant') {
                    mml.setProperty('variantForm', true);
                }
                else if (name.substr(0, 11) !== 'MJX-TeXAtom') {
                    mml.attributes.set('mathvariant', this.fixCalligraphic(name.substr(3)));
                }
            }
            else {
                classList.push(name);
            }
        }
        if (classList.length) {
            mml.attributes.set('class', classList.join(' '));
        }
    }
    fixCalligraphic(variant) {
        return variant.replace(/caligraphic/, 'calligraphic');
    }
    markMrows(mml) {
        if (mml.isKind('mrow') && !mml.isInferred && mml.childNodes.length >= 2) {
            let first = mml.childNodes[0];
            let last = mml.childNodes[mml.childNodes.length - 1];
            if (first.isKind('mo') && first.attributes.get('fence') && first.attributes.get('stretchy') &&
                last.isKind('mo') && last.attributes.get('fence') && last.attributes.get('stretchy')) {
                if (first.childNodes.length) {
                    mml.setProperty('open', first.getText());
                }
                if (last.childNodes.length) {
                    mml.setProperty('close', last.getText());
                }
            }
        }
    }
    trimSpace(text) {
        return text.replace(/[\t\n\r]/g, ' ')
            .replace(/^ +/, '')
            .replace(/ +$/, '')
            .replace(/  +/g, ' ');
    }
    error(message) {
        throw new Error(message);
    }
}
exports.MathMLCompile = MathMLCompile;
MathMLCompile.OPTIONS = {
    MmlFactory: null,
    fixMisplacedChildren: true,
    verify: Object.assign({}, MmlNode_js_1.AbstractMmlNode.verifyDefaults),
    translateEntities: true
};
//# sourceMappingURL=MathMLCompile.js.map