"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BraketItem = void 0;
const StackItem_js_1 = require("../StackItem.js");
const MmlNode_js_1 = require("../../../core/MmlTree/MmlNode.js");
const ParseUtil_js_1 = __importDefault(require("../ParseUtil.js"));
class BraketItem extends StackItem_js_1.BaseItem {
    get kind() {
        return 'braket';
    }
    get isOpen() {
        return true;
    }
    checkItem(item) {
        if (item.isKind('close')) {
            return [[this.factory.create('mml', this.toMml())], true];
        }
        if (item.isKind('mml')) {
            this.Push(item.toMml());
            if (this.getProperty('single')) {
                return [[this.toMml()], true];
            }
            return StackItem_js_1.BaseItem.fail;
        }
        return super.checkItem(item);
    }
    toMml() {
        let inner = super.toMml();
        let open = this.getProperty('open');
        let close = this.getProperty('close');
        if (this.getProperty('stretchy')) {
            return ParseUtil_js_1.default.fenced(this.factory.configuration, open, inner, close);
        }
        let attrs = { fence: true, stretchy: false, symmetric: true, texClass: MmlNode_js_1.TEXCLASS.OPEN };
        let openNode = this.create('token', 'mo', attrs, open);
        attrs.texClass = MmlNode_js_1.TEXCLASS.CLOSE;
        let closeNode = this.create('token', 'mo', attrs, close);
        let mrow = this.create('node', 'mrow', [openNode, inner, closeNode], { open: open, close: close, texClass: MmlNode_js_1.TEXCLASS.INNER });
        return mrow;
    }
}
exports.BraketItem = BraketItem;
//# sourceMappingURL=BraketItems.js.map