"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CasesConfiguration = exports.CasesMethods = exports.CasesTags = exports.CasesBeginItem = void 0;
const Configuration_js_1 = require("../Configuration.js");
const SymbolMap_js_1 = require("../SymbolMap.js");
const ParseUtil_js_1 = __importDefault(require("../ParseUtil.js"));
const BaseMethods_js_1 = __importDefault(require("../base/BaseMethods.js"));
const TexError_js_1 = __importDefault(require("../TexError.js"));
const BaseItems_js_1 = require("../base/BaseItems.js");
const AmsConfiguration_js_1 = require("../ams/AmsConfiguration.js");
const EmpheqUtil_js_1 = require("../empheq/EmpheqUtil.js");
class CasesBeginItem extends BaseItems_js_1.BeginItem {
    get kind() {
        return 'cases-begin';
    }
    checkItem(item) {
        if (item.isKind('end') && item.getName() === this.getName()) {
            if (this.getProperty('end')) {
                this.setProperty('end', false);
                return [[], true];
            }
        }
        return super.checkItem(item);
    }
}
exports.CasesBeginItem = CasesBeginItem;
class CasesTags extends AmsConfiguration_js_1.AmsTags {
    constructor() {
        super(...arguments);
        this.subcounter = 0;
    }
    start(env, taggable, defaultTags) {
        this.subcounter = 0;
        super.start(env, taggable, defaultTags);
    }
    autoTag() {
        if (this.currentTag.tag != null)
            return;
        if (this.currentTag.env === 'subnumcases') {
            if (this.subcounter === 0)
                this.counter++;
            this.subcounter++;
            this.tag(this.formatNumber(this.counter, this.subcounter), false);
        }
        else {
            if (this.subcounter === 0 || this.currentTag.env !== 'numcases-left')
                this.counter++;
            this.tag(this.formatNumber(this.counter), false);
        }
    }
    formatNumber(n, m = null) {
        return n.toString() + (m === null ? '' : String.fromCharCode(0x60 + m));
    }
}
exports.CasesTags = CasesTags;
exports.CasesMethods = {
    NumCases(parser, begin) {
        if (parser.stack.env.closing === begin.getName()) {
            delete parser.stack.env.closing;
            parser.Push(parser.itemFactory.create('end').setProperty('name', begin.getName()));
            const cases = parser.stack.Top();
            const table = cases.Last;
            const original = ParseUtil_js_1.default.copyNode(table, parser);
            const left = cases.getProperty('left');
            EmpheqUtil_js_1.EmpheqUtil.left(table, original, left + '\\empheqlbrace\\,', parser, 'numcases-left');
            parser.Push(parser.itemFactory.create('end').setProperty('name', begin.getName()));
            return null;
        }
        else {
            const left = parser.GetArgument('\\begin{' + begin.getName() + '}');
            begin.setProperty('left', left);
            const array = BaseMethods_js_1.default.EqnArray(parser, begin, true, true, 'll');
            array.arraydef.displaystyle = false;
            array.arraydef.rowspacing = '.2em';
            array.setProperty('numCases', true);
            parser.Push(begin);
            return array;
        }
    },
    Entry(parser, name) {
        if (!parser.stack.Top().getProperty('numCases')) {
            return BaseMethods_js_1.default.Entry(parser, name);
        }
        parser.Push(parser.itemFactory.create('cell').setProperties({ isEntry: true, name: name }));
        const tex = parser.string;
        let braces = 0, i = parser.i, m = tex.length;
        while (i < m) {
            const c = tex.charAt(i);
            if (c === '{') {
                braces++;
                i++;
            }
            else if (c === '}') {
                if (braces === 0) {
                    break;
                }
                else {
                    braces--;
                    i++;
                }
            }
            else if (c === '&' && braces === 0) {
                throw new TexError_js_1.default('ExtraCasesAlignTab', 'Extra alignment tab in text for numcase environment');
            }
            else if (c === '\\' && braces === 0) {
                const cs = (tex.slice(i + 1).match(/^[a-z]+|./i) || [])[0];
                if (cs === '\\' || cs === 'cr' || cs === 'end' || cs === 'label') {
                    break;
                }
                else {
                    i += cs.length;
                }
            }
            else {
                i++;
            }
        }
        const text = tex.substr(parser.i, i - parser.i).replace(/^\s*/, '');
        parser.PushAll(ParseUtil_js_1.default.internalMath(parser, text, 0));
        parser.i = i;
    }
};
new SymbolMap_js_1.EnvironmentMap('cases-env', EmpheqUtil_js_1.EmpheqUtil.environment, {
    numcases: ['NumCases', 'cases'],
    subnumcases: ['NumCases', 'cases']
}, exports.CasesMethods);
new SymbolMap_js_1.MacroMap('cases-macros', {
    '&': 'Entry'
}, exports.CasesMethods);
exports.CasesConfiguration = Configuration_js_1.Configuration.create('cases', {
    handler: {
        environment: ['cases-env'],
        character: ['cases-macros']
    },
    items: {
        [CasesBeginItem.prototype.kind]: CasesBeginItem
    },
    tags: { 'cases': CasesTags }
});
//# sourceMappingURL=CasesConfiguration.js.map