"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CenternotConfiguration = exports.filterCenterOver = void 0;
const Configuration_js_1 = require("../Configuration.js");
const TexParser_js_1 = __importDefault(require("../TexParser.js"));
const NodeUtil_js_1 = __importDefault(require("../NodeUtil.js"));
const SymbolMap_js_1 = require("../SymbolMap.js");
const BaseMethods_js_1 = __importDefault(require("../base/BaseMethods.js"));
new SymbolMap_js_1.CommandMap('centernot', {
    centerOver: 'CenterOver',
    centernot: ['Macro', '\\centerOver{#1}{{\u29F8}}', 1]
}, {
    CenterOver(parser, name) {
        const arg = '{' + parser.GetArgument(name) + '}';
        const over = parser.ParseArg(name);
        const base = new TexParser_js_1.default(arg, parser.stack.env, parser.configuration).mml();
        let mml = parser.create('node', 'TeXAtom', [
            new TexParser_js_1.default(arg, parser.stack.env, parser.configuration).mml(),
            parser.create('node', 'mpadded', [
                parser.create('node', 'mpadded', [over], { width: 0, lspace: '-.5width' }),
                parser.create('node', 'mphantom', [base])
            ], { width: 0, lspace: '-.5width' })
        ]);
        parser.configuration.addNode('centerOver', base);
        parser.Push(mml);
    },
    Macro: BaseMethods_js_1.default.Macro
});
function filterCenterOver({ data }) {
    for (const base of data.getList('centerOver')) {
        const texClass = NodeUtil_js_1.default.getTexClass(base.childNodes[0].childNodes[0]);
        if (texClass !== null) {
            NodeUtil_js_1.default.setProperties(base.parent.parent.parent.parent.parent.parent, { texClass });
        }
    }
}
exports.filterCenterOver = filterCenterOver;
exports.CenternotConfiguration = Configuration_js_1.Configuration.create('centernot', {
    handler: { macro: ['centernot'] },
    postprocessors: [filterCenterOver]
});
//# sourceMappingURL=CenternotConfiguration.js.map