"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorMethods = void 0;
const NodeUtil_js_1 = __importDefault(require("../NodeUtil.js"));
const ParseUtil_js_1 = __importDefault(require("../ParseUtil.js"));
function padding(colorPadding) {
    const pad = `+${colorPadding}`;
    const unit = colorPadding.replace(/^.*?([a-z]*)$/, '$1');
    const pad2 = 2 * parseFloat(pad);
    return {
        width: `+${pad2}${unit}`,
        height: pad,
        depth: pad,
        lspace: colorPadding,
    };
}
exports.ColorMethods = {};
exports.ColorMethods.Color = function (parser, name) {
    const model = parser.GetBrackets(name, '');
    const colorDef = parser.GetArgument(name);
    const colorModel = parser.configuration.packageData.get('color').model;
    const color = colorModel.getColor(model, colorDef);
    const style = parser.itemFactory.create('style')
        .setProperties({ styles: { mathcolor: color } });
    parser.stack.env['color'] = color;
    parser.Push(style);
};
exports.ColorMethods.TextColor = function (parser, name) {
    const model = parser.GetBrackets(name, '');
    const colorDef = parser.GetArgument(name);
    const colorModel = parser.configuration.packageData.get('color').model;
    const color = colorModel.getColor(model, colorDef);
    const old = parser.stack.env['color'];
    parser.stack.env['color'] = color;
    const math = parser.ParseArg(name);
    if (old) {
        parser.stack.env['color'] = old;
    }
    else {
        delete parser.stack.env['color'];
    }
    const node = parser.create('node', 'mstyle', [math], { mathcolor: color });
    parser.Push(node);
};
exports.ColorMethods.DefineColor = function (parser, name) {
    const cname = parser.GetArgument(name);
    const model = parser.GetArgument(name);
    const def = parser.GetArgument(name);
    const colorModel = parser.configuration.packageData.get('color').model;
    colorModel.defineColor(model, cname, def);
};
exports.ColorMethods.ColorBox = function (parser, name) {
    const cname = parser.GetArgument(name);
    const math = ParseUtil_js_1.default.internalMath(parser, parser.GetArgument(name));
    const colorModel = parser.configuration.packageData.get('color').model;
    const node = parser.create('node', 'mpadded', math, {
        mathbackground: colorModel.getColor('named', cname)
    });
    NodeUtil_js_1.default.setProperties(node, padding(parser.options.color.padding));
    parser.Push(node);
};
exports.ColorMethods.FColorBox = function (parser, name) {
    const fname = parser.GetArgument(name);
    const cname = parser.GetArgument(name);
    const math = ParseUtil_js_1.default.internalMath(parser, parser.GetArgument(name));
    const options = parser.options.color;
    const colorModel = parser.configuration.packageData.get('color').model;
    const node = parser.create('node', 'mpadded', math, {
        mathbackground: colorModel.getColor('named', cname),
        style: `border: ${options.borderWidth} solid ${colorModel.getColor('named', fname)}`
    });
    NodeUtil_js_1.default.setProperties(node, padding(options.padding));
    parser.Push(node);
};
//# sourceMappingURL=ColorMethods.js.map