"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorModel = void 0;
const TexError_js_1 = __importDefault(require("../TexError.js"));
const ColorConstants_js_1 = require("./ColorConstants.js");
const ColorModelProcessors = new Map();
class ColorModel {
    constructor() {
        this.userColors = new Map();
    }
    normalizeColor(model, def) {
        if (!model || model === 'named') {
            return def;
        }
        if (ColorModelProcessors.has(model)) {
            const modelProcessor = ColorModelProcessors.get(model);
            return modelProcessor(def);
        }
        throw new TexError_js_1.default('UndefinedColorModel', 'Color model \'%1\' not defined', model);
    }
    getColor(model, def) {
        if (!model || model === 'named') {
            return this.getColorByName(def);
        }
        return this.normalizeColor(model, def);
    }
    getColorByName(name) {
        if (this.userColors.has(name)) {
            return this.userColors.get(name);
        }
        if (ColorConstants_js_1.COLORS.has(name)) {
            return ColorConstants_js_1.COLORS.get(name);
        }
        return name;
    }
    defineColor(model, name, def) {
        const normalized = this.normalizeColor(model, def);
        this.userColors.set(name, normalized);
    }
}
exports.ColorModel = ColorModel;
ColorModelProcessors.set('rgb', function (rgb) {
    const rgbParts = rgb.trim().split(/\s*,\s*/);
    let RGB = '#';
    if (rgbParts.length !== 3) {
        throw new TexError_js_1.default('ModelArg1', 'Color values for the %1 model require 3 numbers', 'rgb');
    }
    for (const rgbPart of rgbParts) {
        if (!rgbPart.match(/^(\d+(\.\d*)?|\.\d+)$/)) {
            throw new TexError_js_1.default('InvalidDecimalNumber', 'Invalid decimal number');
        }
        const n = parseFloat(rgbPart);
        if (n < 0 || n > 1) {
            throw new TexError_js_1.default('ModelArg2', 'Color values for the %1 model must be between %2 and %3', 'rgb', '0', '1');
        }
        let pn = Math.floor(n * 255).toString(16);
        if (pn.length < 2) {
            pn = '0' + pn;
        }
        RGB += pn;
    }
    return RGB;
});
ColorModelProcessors.set('RGB', function (rgb) {
    const rgbParts = rgb.trim().split(/\s*,\s*/);
    let RGB = '#';
    if (rgbParts.length !== 3) {
        throw new TexError_js_1.default('ModelArg1', 'Color values for the %1 model require 3 numbers', 'RGB');
    }
    for (const rgbPart of rgbParts) {
        if (!rgbPart.match(/^\d+$/)) {
            throw new TexError_js_1.default('InvalidNumber', 'Invalid number');
        }
        const n = parseInt(rgbPart);
        if (n > 255) {
            throw new TexError_js_1.default('ModelArg2', 'Color values for the %1 model must be between %2 and %3', 'RGB', '0', '255');
        }
        let pn = n.toString(16);
        if (pn.length < 2) {
            pn = '0' + pn;
        }
        RGB += pn;
    }
    return RGB;
});
ColorModelProcessors.set('gray', function (gray) {
    if (!gray.match(/^\s*(\d+(\.\d*)?|\.\d+)\s*$/)) {
        throw new TexError_js_1.default('InvalidDecimalNumber', 'Invalid decimal number');
    }
    const n = parseFloat(gray);
    if (n < 0 || n > 1) {
        throw new TexError_js_1.default('ModelArg2', 'Color values for the %1 model must be between %2 and %3', 'gray', '0', '1');
    }
    let pn = Math.floor(n * 255).toString(16);
    if (pn.length < 2) {
        pn = '0' + pn;
    }
    return `#${pn}${pn}${pn}`;
});
//# sourceMappingURL=ColorUtil.js.map