"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigMacrosConfiguration = void 0;
const Configuration_js_1 = require("../Configuration.js");
const Options_js_1 = require("../../../util/Options.js");
const SymbolMap_js_1 = require("../SymbolMap.js");
const ParseMethods_js_1 = __importDefault(require("../ParseMethods.js"));
const Symbol_js_1 = require("../Symbol.js");
const NewcommandMethods_js_1 = __importDefault(require("../newcommand/NewcommandMethods.js"));
const NewcommandItems_js_1 = require("../newcommand/NewcommandItems.js");
const MACROSMAP = 'configmacros-map';
const ENVIRONMENTMAP = 'configmacros-env-map';
function configmacrosInit(config) {
    new SymbolMap_js_1.CommandMap(MACROSMAP, {}, {});
    new SymbolMap_js_1.EnvironmentMap(ENVIRONMENTMAP, ParseMethods_js_1.default.environment, {}, {});
    config.append(Configuration_js_1.Configuration.local({
        handler: {
            macro: [MACROSMAP],
            environment: [ENVIRONMENTMAP]
        },
        priority: 3
    }));
}
function configmacrosConfig(_config, jax) {
    configMacros(jax);
    configEnvironments(jax);
}
function configMacros(jax) {
    const handler = jax.parseOptions.handlers.retrieve(MACROSMAP);
    const macros = jax.parseOptions.options.macros;
    for (const cs of Object.keys(macros)) {
        const def = (typeof macros[cs] === 'string' ? [macros[cs]] : macros[cs]);
        const macro = Array.isArray(def[2]) ?
            new Symbol_js_1.Macro(cs, NewcommandMethods_js_1.default.MacroWithTemplate, def.slice(0, 2).concat(def[2])) :
            new Symbol_js_1.Macro(cs, NewcommandMethods_js_1.default.Macro, def);
        handler.add(cs, macro);
    }
}
function configEnvironments(jax) {
    const handler = jax.parseOptions.handlers.retrieve(ENVIRONMENTMAP);
    const environments = jax.parseOptions.options.environments;
    for (const env of Object.keys(environments)) {
        handler.add(env, new Symbol_js_1.Macro(env, NewcommandMethods_js_1.default.BeginEnv, [true].concat(environments[env])));
    }
}
exports.ConfigMacrosConfiguration = Configuration_js_1.Configuration.create('configmacros', {
    init: configmacrosInit,
    config: configmacrosConfig,
    items: {
        [NewcommandItems_js_1.BeginEnvItem.prototype.kind]: NewcommandItems_js_1.BeginEnvItem,
    },
    options: {
        macros: (0, Options_js_1.expandable)({}),
        environments: (0, Options_js_1.expandable)({})
    }
});
//# sourceMappingURL=ConfigMacrosConfiguration.js.map