"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MathtoolsConfiguration = exports.fixPrescripts = exports.PAIREDDELIMS = void 0;
const Configuration_js_1 = require("../Configuration.js");
const SymbolMap_js_1 = require("../SymbolMap.js");
const NodeUtil_js_1 = __importDefault(require("../NodeUtil.js"));
const Options_js_1 = require("../../../util/Options.js");
require("./MathtoolsMappings.js");
const MathtoolsUtil_js_1 = require("./MathtoolsUtil.js");
const MathtoolsTags_js_1 = require("./MathtoolsTags.js");
const MathtoolsItems_js_1 = require("./MathtoolsItems.js");
exports.PAIREDDELIMS = 'mathtools-paired-delims';
function initMathtools(config) {
    new SymbolMap_js_1.CommandMap(exports.PAIREDDELIMS, {}, {});
    config.append(Configuration_js_1.Configuration.local({ handler: { macro: [exports.PAIREDDELIMS] }, priority: -5 }));
}
function configMathtools(config, jax) {
    const parser = jax.parseOptions;
    const pairedDelims = parser.options.mathtools.pairedDelimiters;
    for (const cs of Object.keys(pairedDelims)) {
        MathtoolsUtil_js_1.MathtoolsUtil.addPairedDelims(parser, cs, pairedDelims[cs]);
    }
    (0, MathtoolsTags_js_1.MathtoolsTagFormat)(config, jax);
}
function fixPrescripts({ data }) {
    for (const node of data.getList('mmultiscripts')) {
        if (!node.getProperty('fixPrescript'))
            continue;
        const childNodes = NodeUtil_js_1.default.getChildren(node);
        let n = 0;
        for (const i of [1, 2]) {
            if (!childNodes[i]) {
                NodeUtil_js_1.default.setChild(node, i, data.nodeFactory.create('node', 'none'));
                n++;
            }
        }
        for (const i of [4, 5]) {
            if (NodeUtil_js_1.default.isType(childNodes[i], 'mrow') && NodeUtil_js_1.default.getChildren(childNodes[i]).length === 0) {
                NodeUtil_js_1.default.setChild(node, i, data.nodeFactory.create('node', 'none'));
            }
        }
        if (n === 2) {
            childNodes.splice(1, 2);
        }
    }
}
exports.fixPrescripts = fixPrescripts;
exports.MathtoolsConfiguration = Configuration_js_1.Configuration.create('mathtools', {
    handler: {
        macro: ['mathtools-macros', 'mathtools-delimiters'],
        environment: ['mathtools-environments'],
        delimiter: ['mathtools-delimiters'],
        character: ['mathtools-characters']
    },
    items: {
        [MathtoolsItems_js_1.MultlinedItem.prototype.kind]: MathtoolsItems_js_1.MultlinedItem
    },
    init: initMathtools,
    config: configMathtools,
    postprocessors: [[fixPrescripts, -6]],
    options: {
        mathtools: {
            'multlinegap': '1em',
            'multlined-pos': 'c',
            'firstline-afterskip': '',
            'lastline-preskip': '',
            'smallmatrix-align': 'c',
            'shortvdotsadjustabove': '.2em',
            'shortvdotsadjustbelow': '.2em',
            'centercolon': false,
            'centercolon-offset': '.04em',
            'thincolon-dx': '-.04em',
            'thincolon-dw': '-.08em',
            'use-unicode': false,
            'prescript-sub-format': '',
            'prescript-sup-format': '',
            'prescript-arg-format': '',
            'allow-mathtoolsset': true,
            pairedDelimiters: (0, Options_js_1.expandable)({}),
            tagforms: (0, Options_js_1.expandable)({}),
        }
    }
});
//# sourceMappingURL=MathtoolsConfiguration.js.map