"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MathtoolsTagFormat = void 0;
const TexError_js_1 = __importDefault(require("../TexError.js"));
const Tags_js_1 = require("../Tags.js");
let tagID = 0;
function MathtoolsTagFormat(config, jax) {
    const tags = jax.parseOptions.options.tags;
    if (tags !== 'base' && config.tags.hasOwnProperty(tags)) {
        Tags_js_1.TagsFactory.add(tags, config.tags[tags]);
    }
    const TagClass = Tags_js_1.TagsFactory.create(jax.parseOptions.options.tags).constructor;
    class TagFormat extends TagClass {
        constructor() {
            super();
            this.mtFormats = new Map();
            this.mtCurrent = null;
            const forms = jax.parseOptions.options.mathtools.tagforms;
            for (const form of Object.keys(forms)) {
                if (!Array.isArray(forms[form]) || forms[form].length !== 3) {
                    throw new TexError_js_1.default('InvalidTagFormDef', 'The tag form definition for "%1" should be an array fo three strings', form);
                }
                this.mtFormats.set(form, forms[form]);
            }
        }
        formatTag(tag) {
            if (this.mtCurrent) {
                const [left, right, format] = this.mtCurrent;
                return (format ? `${left}${format}{${tag}}${right}` : `${left}${tag}${right}`);
            }
            return super.formatTag(tag);
        }
    }
    tagID++;
    const tagName = 'MathtoolsTags-' + tagID;
    Tags_js_1.TagsFactory.add(tagName, TagFormat);
    jax.parseOptions.options.tags = tagName;
}
exports.MathtoolsTagFormat = MathtoolsTagFormat;
//# sourceMappingURL=MathtoolsTags.js.map