"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoOpen = void 0;
const StackItem_js_1 = require("../StackItem.js");
const ParseUtil_js_1 = __importDefault(require("../ParseUtil.js"));
const NodeUtil_js_1 = __importDefault(require("../NodeUtil.js"));
const TexParser_js_1 = __importDefault(require("../TexParser.js"));
class AutoOpen extends StackItem_js_1.BaseItem {
    constructor() {
        super(...arguments);
        this.openCount = 0;
    }
    get kind() {
        return 'auto open';
    }
    get isOpen() {
        return true;
    }
    toMml() {
        let parser = this.factory.configuration.parser;
        let right = this.getProperty('right');
        if (this.getProperty('smash')) {
            let mml = super.toMml();
            const smash = parser.create('node', 'mpadded', [mml], { height: 0, depth: 0 });
            this.Clear();
            this.Push(parser.create('node', 'TeXAtom', [smash]));
        }
        if (right) {
            this.Push(new TexParser_js_1.default(right, parser.stack.env, parser.configuration).mml());
        }
        let mml = ParseUtil_js_1.default.fenced(this.factory.configuration, this.getProperty('open'), super.toMml(), this.getProperty('close'), this.getProperty('big'));
        NodeUtil_js_1.default.removeProperties(mml, 'open', 'close', 'texClass');
        return mml;
    }
    checkItem(item) {
        if (item.isKind('mml') && item.Size() === 1) {
            const mml = item.toMml();
            if (mml.isKind('mo') && mml.getText() === this.getProperty('open')) {
                this.openCount++;
            }
        }
        let close = item.getProperty('autoclose');
        if (close && close === this.getProperty('close') && !this.openCount--) {
            if (this.getProperty('ignore')) {
                this.Clear();
                return [[], true];
            }
            return [[this.toMml()], true];
        }
        return super.checkItem(item);
    }
}
exports.AutoOpen = AutoOpen;
AutoOpen.errors = Object.assign(Object.create(StackItem_js_1.BaseItem.errors), {
    'stop': ['ExtraOrMissingDelims', 'Extra open or missing close delimiter']
});
//# sourceMappingURL=PhysicsItems.js.map