"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CHTML = void 0;
const OutputJax_js_1 = require("./common/OutputJax.js");
const StyleList_js_1 = require("../util/StyleList.js");
const WrapperFactory_js_1 = require("./chtml/WrapperFactory.js");
const Usage_js_1 = require("./chtml/Usage.js");
const tex_js_1 = require("./chtml/fonts/tex.js");
const LENGTHS = __importStar(require("../util/lengths.js"));
const string_js_1 = require("../util/string.js");
class CHTML extends OutputJax_js_1.CommonOutputJax {
    constructor(options = null) {
        super(options, WrapperFactory_js_1.CHTMLWrapperFactory, tex_js_1.TeXFont);
        this.chtmlStyles = null;
        this.font.adaptiveCSS(this.options.adaptiveCSS);
        this.wrapperUsage = new Usage_js_1.Usage();
    }
    escaped(math, html) {
        this.setDocument(html);
        return this.html('span', {}, [this.text(math.math)]);
    }
    styleSheet(html) {
        if (this.chtmlStyles) {
            if (this.options.adaptiveCSS) {
                const styles = new StyleList_js_1.CssStyles();
                this.addWrapperStyles(styles);
                this.updateFontStyles(styles);
                this.adaptor.insertRules(this.chtmlStyles, styles.getStyleRules());
            }
            return this.chtmlStyles;
        }
        const sheet = this.chtmlStyles = super.styleSheet(html);
        this.adaptor.setAttribute(sheet, 'id', CHTML.STYLESHEETID);
        this.wrapperUsage.update();
        return sheet;
    }
    updateFontStyles(styles) {
        styles.addStyles(this.font.updateStyles({}));
    }
    addWrapperStyles(styles) {
        if (!this.options.adaptiveCSS) {
            super.addWrapperStyles(styles);
            return;
        }
        for (const kind of this.wrapperUsage.update()) {
            const wrapper = this.factory.getNodeClass(kind);
            wrapper && this.addClassStyles(wrapper, styles);
        }
    }
    addClassStyles(wrapper, styles) {
        const CLASS = wrapper;
        if (CLASS.autoStyle && CLASS.kind !== 'unknown') {
            styles.addStyles({
                ['mjx-' + CLASS.kind]: {
                    display: 'inline-block',
                    'text-align': 'left'
                }
            });
        }
        this.wrapperUsage.add(CLASS.kind);
        super.addClassStyles(wrapper, styles);
    }
    processMath(math, parent) {
        this.factory.wrap(math).toCHTML(parent);
    }
    clearCache() {
        this.cssStyles.clear();
        this.font.clearCache();
        this.wrapperUsage.clear();
        this.chtmlStyles = null;
    }
    reset() {
        this.clearCache();
    }
    unknownText(text, variant, width = null) {
        const styles = {};
        const scale = 100 / this.math.metrics.scale;
        if (scale !== 100) {
            styles['font-size'] = this.fixed(scale, 1) + '%';
            styles.padding = LENGTHS.em(75 / scale) + ' 0 ' + LENGTHS.em(20 / scale) + ' 0';
        }
        if (variant !== '-explicitFont') {
            const c = (0, string_js_1.unicodeChars)(text);
            if (c.length !== 1 || c[0] < 0x1D400 || c[0] > 0x1D7FF) {
                this.cssFontStyles(this.font.getCssFont(variant), styles);
            }
        }
        if (width !== null) {
            const metrics = this.math.metrics;
            styles.width = Math.round(width * metrics.em * metrics.scale) + 'px';
        }
        return this.html('mjx-utext', { variant: variant, style: styles }, [this.text(text)]);
    }
    measureTextNode(textNode) {
        const adaptor = this.adaptor;
        const text = adaptor.clone(textNode);
        adaptor.setStyle(text, 'font-family', adaptor.getStyle(text, 'font-family').replace(/MJXZERO, /g, ''));
        const style = { position: 'absolute', 'white-space': 'nowrap' };
        const node = this.html('mjx-measure-text', { style }, [text]);
        adaptor.append(adaptor.parent(this.math.start.node), this.container);
        adaptor.append(this.container, node);
        let w = adaptor.nodeSize(text, this.math.metrics.em)[0] / this.math.metrics.scale;
        adaptor.remove(this.container);
        adaptor.remove(node);
        return { w: w, h: .75, d: .2 };
    }
}
exports.CHTML = CHTML;
CHTML.NAME = 'CHTML';
CHTML.OPTIONS = Object.assign(Object.assign({}, OutputJax_js_1.CommonOutputJax.OPTIONS), { adaptiveCSS: true, matchFontHeight: true });
CHTML.commonStyles = {
    'mjx-container[jax="CHTML"]': { 'line-height': 0 },
    'mjx-container [space="1"]': { 'margin-left': '.111em' },
    'mjx-container [space="2"]': { 'margin-left': '.167em' },
    'mjx-container [space="3"]': { 'margin-left': '.222em' },
    'mjx-container [space="4"]': { 'margin-left': '.278em' },
    'mjx-container [space="5"]': { 'margin-left': '.333em' },
    'mjx-container [rspace="1"]': { 'margin-right': '.111em' },
    'mjx-container [rspace="2"]': { 'margin-right': '.167em' },
    'mjx-container [rspace="3"]': { 'margin-right': '.222em' },
    'mjx-container [rspace="4"]': { 'margin-right': '.278em' },
    'mjx-container [rspace="5"]': { 'margin-right': '.333em' },
    'mjx-container [size="s"]': { 'font-size': '70.7%' },
    'mjx-container [size="ss"]': { 'font-size': '50%' },
    'mjx-container [size="Tn"]': { 'font-size': '60%' },
    'mjx-container [size="sm"]': { 'font-size': '85%' },
    'mjx-container [size="lg"]': { 'font-size': '120%' },
    'mjx-container [size="Lg"]': { 'font-size': '144%' },
    'mjx-container [size="LG"]': { 'font-size': '173%' },
    'mjx-container [size="hg"]': { 'font-size': '207%' },
    'mjx-container [size="HG"]': { 'font-size': '249%' },
    'mjx-container [width="full"]': { width: '100%' },
    'mjx-box': { display: 'inline-block' },
    'mjx-block': { display: 'block' },
    'mjx-itable': { display: 'inline-table' },
    'mjx-row': { display: 'table-row' },
    'mjx-row > *': { display: 'table-cell' },
    'mjx-mtext': {
        display: 'inline-block'
    },
    'mjx-mstyle': {
        display: 'inline-block'
    },
    'mjx-merror': {
        display: 'inline-block',
        color: 'red',
        'background-color': 'yellow'
    },
    'mjx-mphantom': {
        visibility: 'hidden'
    },
    '_::-webkit-full-page-media, _:future, :root mjx-container': {
        'will-change': 'opacity'
    }
};
CHTML.STYLESHEETID = 'MJX-CHTML-styles';
//# sourceMappingURL=chtml.js.map