"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddCSS = exports.CHTMLFontData = void 0;
const FontData_js_1 = require("../common/FontData.js");
const Usage_js_1 = require("./Usage.js");
const lengths_js_1 = require("../../util/lengths.js");
__exportStar(require("../common/FontData.js"), exports);
class CHTMLFontData extends FontData_js_1.FontData {
    constructor() {
        super(...arguments);
        this.charUsage = new Usage_js_1.Usage();
        this.delimUsage = new Usage_js_1.Usage();
    }
    static charOptions(font, n) {
        return super.charOptions(font, n);
    }
    adaptiveCSS(adapt) {
        this.options.adaptiveCSS = adapt;
    }
    clearCache() {
        if (this.options.adaptiveCSS) {
            this.charUsage.clear();
            this.delimUsage.clear();
        }
    }
    createVariant(name, inherit = null, link = null) {
        super.createVariant(name, inherit, link);
        let CLASS = this.constructor;
        this.variant[name].classes = CLASS.defaultVariantClasses[name];
        this.variant[name].letter = CLASS.defaultVariantLetters[name];
    }
    defineChars(name, chars) {
        super.defineChars(name, chars);
        const letter = this.variant[name].letter;
        for (const n of Object.keys(chars)) {
            const options = CHTMLFontData.charOptions(chars, parseInt(n));
            if (options.f === undefined) {
                options.f = letter;
            }
        }
    }
    get styles() {
        const CLASS = this.constructor;
        const styles = Object.assign({}, CLASS.defaultStyles);
        this.addFontURLs(styles, CLASS.defaultFonts, this.options.fontURL);
        if (this.options.adaptiveCSS) {
            this.updateStyles(styles);
        }
        else {
            this.allStyles(styles);
        }
        return styles;
    }
    updateStyles(styles) {
        for (const N of this.delimUsage.update()) {
            this.addDelimiterStyles(styles, N, this.delimiters[N]);
        }
        for (const [name, N] of this.charUsage.update()) {
            const variant = this.variant[name];
            this.addCharStyles(styles, variant.letter, N, variant.chars[N]);
        }
        return styles;
    }
    allStyles(styles) {
        for (const n of Object.keys(this.delimiters)) {
            const N = parseInt(n);
            this.addDelimiterStyles(styles, N, this.delimiters[N]);
        }
        for (const name of Object.keys(this.variant)) {
            const variant = this.variant[name];
            const vletter = variant.letter;
            for (const n of Object.keys(variant.chars)) {
                const N = parseInt(n);
                const char = variant.chars[N];
                if ((char[3] || {}).smp)
                    continue;
                if (char.length < 4) {
                    char[3] = {};
                }
                this.addCharStyles(styles, vletter, N, char);
            }
        }
    }
    addFontURLs(styles, fonts, url) {
        for (const name of Object.keys(fonts)) {
            const font = Object.assign({}, fonts[name]);
            font.src = font.src.replace(/%%URL%%/, url);
            styles[name] = font;
        }
    }
    addDelimiterStyles(styles, n, data) {
        let c = this.charSelector(n);
        if (data.c && data.c !== n) {
            c = this.charSelector(data.c);
            styles['.mjx-stretched mjx-c' + c + '::before'] = {
                content: this.charContent(data.c)
            };
        }
        if (!data.stretch)
            return;
        if (data.dir === 1) {
            this.addDelimiterVStyles(styles, c, data);
        }
        else {
            this.addDelimiterHStyles(styles, c, data);
        }
    }
    addDelimiterVStyles(styles, c, data) {
        const HDW = data.HDW;
        const [beg, ext, end, mid] = data.stretch;
        const Hb = this.addDelimiterVPart(styles, c, 'beg', beg, HDW);
        this.addDelimiterVPart(styles, c, 'ext', ext, HDW);
        const He = this.addDelimiterVPart(styles, c, 'end', end, HDW);
        const css = {};
        if (mid) {
            const Hm = this.addDelimiterVPart(styles, c, 'mid', mid, HDW);
            css.height = '50%';
            styles['mjx-stretchy-v' + c + ' > mjx-mid'] = {
                'margin-top': this.em(-Hm / 2),
                'margin-bottom': this.em(-Hm / 2)
            };
        }
        if (Hb) {
            css['border-top-width'] = this.em0(Hb - .03);
        }
        if (He) {
            css['border-bottom-width'] = this.em0(He - .03);
            styles['mjx-stretchy-v' + c + ' > mjx-end'] = { 'margin-top': this.em(-He) };
        }
        if (Object.keys(css).length) {
            styles['mjx-stretchy-v' + c + ' > mjx-ext'] = css;
        }
    }
    addDelimiterVPart(styles, c, part, n, HDW) {
        if (!n)
            return 0;
        const data = this.getDelimiterData(n);
        const dw = (HDW[2] - data[2]) / 2;
        const css = { content: this.charContent(n) };
        if (part !== 'ext') {
            css.padding = this.padding(data, dw);
        }
        else {
            css.width = this.em0(HDW[2]);
            if (dw) {
                css['padding-left'] = this.em0(dw);
            }
        }
        styles['mjx-stretchy-v' + c + ' mjx-' + part + ' mjx-c::before'] = css;
        return data[0] + data[1];
    }
    addDelimiterHStyles(styles, c, data) {
        const [beg, ext, end, mid] = data.stretch;
        const HDW = data.HDW;
        this.addDelimiterHPart(styles, c, 'beg', beg, HDW);
        this.addDelimiterHPart(styles, c, 'ext', ext, HDW);
        this.addDelimiterHPart(styles, c, 'end', end, HDW);
        if (mid) {
            this.addDelimiterHPart(styles, c, 'mid', mid, HDW);
            styles['mjx-stretchy-h' + c + ' > mjx-ext'] = { width: '50%' };
        }
    }
    addDelimiterHPart(styles, c, part, n, HDW) {
        if (!n)
            return;
        const data = this.getDelimiterData(n);
        const options = data[3];
        const css = { content: (options && options.c ? '"' + options.c + '"' : this.charContent(n)) };
        css.padding = this.padding(HDW, 0, -HDW[2]);
        styles['mjx-stretchy-h' + c + ' mjx-' + part + ' mjx-c::before'] = css;
    }
    addCharStyles(styles, vletter, n, data) {
        const options = data[3];
        const letter = (options.f !== undefined ? options.f : vletter);
        const selector = 'mjx-c' + this.charSelector(n) + (letter ? '.TEX-' + letter : '');
        styles[selector + '::before'] = {
            padding: this.padding(data, 0, options.ic || 0),
            content: (options.c != null ? '"' + options.c + '"' : this.charContent(n))
        };
    }
    getDelimiterData(n) {
        return this.getChar('-smallop', n);
    }
    em(n) {
        return (0, lengths_js_1.em)(n);
    }
    em0(n) {
        return (0, lengths_js_1.em)(Math.max(0, n));
    }
    padding([h, d, w], dw = 0, ic = 0) {
        return [h, w + ic, d, dw].map(this.em0).join(' ');
    }
    charContent(n) {
        return '"' + (n >= 0x20 && n <= 0x7E && n !== 0x22 && n !== 0x27 && n !== 0x5C ?
            String.fromCharCode(n) : '\\' + n.toString(16).toUpperCase()) + '"';
    }
    charSelector(n) {
        return '.mjx-c' + n.toString(16).toUpperCase();
    }
}
exports.CHTMLFontData = CHTMLFontData;
CHTMLFontData.OPTIONS = Object.assign(Object.assign({}, FontData_js_1.FontData.OPTIONS), { fontURL: 'js/output/chtml/fonts/tex-woff-v2' });
CHTMLFontData.JAX = 'CHTML';
CHTMLFontData.defaultVariantClasses = {};
CHTMLFontData.defaultVariantLetters = {};
CHTMLFontData.defaultStyles = {
    'mjx-c::before': {
        display: 'block',
        width: 0
    }
};
CHTMLFontData.defaultFonts = {
    '@font-face /* 0 */': {
        'font-family': 'MJXZERO',
        src: 'url("%%URL%%/MathJax_Zero.woff") format("woff")'
    }
};
function AddCSS(font, options) {
    for (const c of Object.keys(options)) {
        const n = parseInt(c);
        Object.assign(FontData_js_1.FontData.charOptions(font, n), options[n]);
    }
    return font;
}
exports.AddCSS = AddCSS;
//# sourceMappingURL=FontData.js.map