"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CHTMLmaction = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const maction_js_1 = require("../../common/Wrappers/maction.js");
const maction_js_2 = require("../../common/Wrappers/maction.js");
const maction_js_3 = require("../../../core/MmlTree/MmlNodes/maction.js");
class CHTMLmaction extends (0, maction_js_1.CommonMactionMixin)(Wrapper_js_1.CHTMLWrapper) {
    toCHTML(parent) {
        const chtml = this.standardCHTMLnode(parent);
        const child = this.selected;
        child.toCHTML(chtml);
        this.action(this, this.data);
    }
    setEventHandler(type, handler) {
        this.chtml.addEventListener(type, handler);
    }
}
exports.CHTMLmaction = CHTMLmaction;
CHTMLmaction.kind = maction_js_3.MmlMaction.prototype.kind;
CHTMLmaction.styles = {
    'mjx-maction': {
        position: 'relative'
    },
    'mjx-maction > mjx-tool': {
        display: 'none',
        position: 'absolute',
        bottom: 0, right: 0,
        width: 0, height: 0,
        'z-index': 500
    },
    'mjx-tool > mjx-tip': {
        display: 'inline-block',
        padding: '.2em',
        border: '1px solid #888',
        'font-size': '70%',
        'background-color': '#F8F8F8',
        color: 'black',
        'box-shadow': '2px 2px 5px #AAAAAA'
    },
    'mjx-maction[toggle]': {
        cursor: 'pointer'
    },
    'mjx-status': {
        display: 'block',
        position: 'fixed',
        left: '1em',
        bottom: '1em',
        'min-width': '25%',
        padding: '.2em .4em',
        border: '1px solid #888',
        'font-size': '90%',
        'background-color': '#F8F8F8',
        color: 'black'
    }
};
CHTMLmaction.actions = new Map([
    ['toggle', [(node, _data) => {
                node.adaptor.setAttribute(node.chtml, 'toggle', node.node.attributes.get('selection'));
                const math = node.factory.jax.math;
                const document = node.factory.jax.document;
                const mml = node.node;
                node.setEventHandler('click', (event) => {
                    if (!math.end.node) {
                        math.start.node = math.end.node = math.typesetRoot;
                        math.start.n = math.end.n = 0;
                    }
                    mml.nextToggleSelection();
                    math.rerender(document);
                    event.stopPropagation();
                });
            }, {}]],
    ['tooltip', [(node, data) => {
                const tip = node.childNodes[1];
                if (!tip)
                    return;
                if (tip.node.isKind('mtext')) {
                    const text = tip.node.getText();
                    node.adaptor.setAttribute(node.chtml, 'title', text);
                }
                else {
                    const adaptor = node.adaptor;
                    const tool = adaptor.append(node.chtml, node.html('mjx-tool', {
                        style: { bottom: node.em(-node.dy), right: node.em(-node.dx) }
                    }, [node.html('mjx-tip')]));
                    tip.toCHTML(adaptor.firstChild(tool));
                    node.setEventHandler('mouseover', (event) => {
                        data.stopTimers(node, data);
                        const timeout = setTimeout(() => adaptor.setStyle(tool, 'display', 'block'), data.postDelay);
                        data.hoverTimer.set(node, timeout);
                        event.stopPropagation();
                    });
                    node.setEventHandler('mouseout', (event) => {
                        data.stopTimers(node, data);
                        const timeout = setTimeout(() => adaptor.setStyle(tool, 'display', ''), data.clearDelay);
                        data.clearTimer.set(node, timeout);
                        event.stopPropagation();
                    });
                }
            }, maction_js_2.TooltipData]],
    ['statusline', [(node, data) => {
                const tip = node.childNodes[1];
                if (!tip)
                    return;
                if (tip.node.isKind('mtext')) {
                    const adaptor = node.adaptor;
                    const text = tip.node.getText();
                    adaptor.setAttribute(node.chtml, 'statusline', text);
                    node.setEventHandler('mouseover', (event) => {
                        if (data.status === null) {
                            const body = adaptor.body(adaptor.document);
                            data.status = adaptor.append(body, node.html('mjx-status', {}, [node.text(text)]));
                        }
                        event.stopPropagation();
                    });
                    node.setEventHandler('mouseout', (event) => {
                        if (data.status) {
                            adaptor.remove(data.status);
                            data.status = null;
                        }
                        event.stopPropagation();
                    });
                }
            }, {
                status: null
            }]]
]);
//# sourceMappingURL=maction.js.map