"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CHTMLmath = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const math_js_1 = require("../../common/Wrappers/math.js");
const math_js_2 = require("../../../core/MmlTree/MmlNodes/math.js");
const BBox_js_1 = require("../../../util/BBox.js");
class CHTMLmath extends (0, math_js_1.CommonMathMixin)(Wrapper_js_1.CHTMLWrapper) {
    toCHTML(parent) {
        super.toCHTML(parent);
        const chtml = this.chtml;
        const adaptor = this.adaptor;
        const display = (this.node.attributes.get('display') === 'block');
        if (display) {
            adaptor.setAttribute(chtml, 'display', 'true');
            adaptor.setAttribute(parent, 'display', 'true');
            this.handleDisplay(parent);
        }
        else {
            this.handleInline(parent);
        }
        adaptor.addClass(chtml, 'MJX-TEX');
    }
    handleDisplay(parent) {
        const adaptor = this.adaptor;
        const [align, shift] = this.getAlignShift();
        if (align !== 'center') {
            adaptor.setAttribute(parent, 'justify', align);
        }
        if (this.bbox.pwidth === BBox_js_1.BBox.fullWidth) {
            adaptor.setAttribute(parent, 'width', 'full');
            if (this.jax.table) {
                let { L, w, R } = this.jax.table.getOuterBBox();
                if (align === 'right') {
                    R = Math.max(R || -shift, -shift);
                }
                else if (align === 'left') {
                    L = Math.max(L || shift, shift);
                }
                else if (align === 'center') {
                    w += 2 * Math.abs(shift);
                }
                const W = this.em(Math.max(0, L + w + R));
                adaptor.setStyle(parent, 'min-width', W);
                adaptor.setStyle(this.jax.table.chtml, 'min-width', W);
            }
        }
        else {
            this.setIndent(this.chtml, align, shift);
        }
    }
    handleInline(parent) {
        const adaptor = this.adaptor;
        const margin = adaptor.getStyle(this.chtml, 'margin-right');
        if (margin) {
            adaptor.setStyle(this.chtml, 'margin-right', '');
            adaptor.setStyle(parent, 'margin-right', margin);
            adaptor.setStyle(parent, 'width', '0');
        }
    }
    setChildPWidths(recompute, w = null, clear = true) {
        return (this.parent ? super.setChildPWidths(recompute, w, clear) : false);
    }
}
exports.CHTMLmath = CHTMLmath;
CHTMLmath.kind = math_js_2.MmlMath.prototype.kind;
CHTMLmath.styles = {
    'mjx-math': {
        'line-height': 0,
        'text-align': 'left',
        'text-indent': 0,
        'font-style': 'normal',
        'font-weight': 'normal',
        'font-size': '100%',
        'font-size-adjust': 'none',
        'letter-spacing': 'normal',
        'border-collapse': 'collapse',
        'word-wrap': 'normal',
        'word-spacing': 'normal',
        'white-space': 'nowrap',
        'direction': 'ltr',
        'padding': '1px 0'
    },
    'mjx-container[jax="CHTML"][display="true"]': {
        display: 'block',
        'text-align': 'center',
        margin: '1em 0'
    },
    'mjx-container[jax="CHTML"][display="true"][width="full"]': {
        display: 'flex'
    },
    'mjx-container[jax="CHTML"][display="true"] mjx-math': {
        padding: 0
    },
    'mjx-container[jax="CHTML"][justify="left"]': {
        'text-align': 'left'
    },
    'mjx-container[jax="CHTML"][justify="right"]': {
        'text-align': 'right'
    }
};
//# sourceMappingURL=math.js.map