"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CHTMLmo = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const mo_js_1 = require("../../common/Wrappers/mo.js");
const mo_js_2 = require("../../../core/MmlTree/MmlNodes/mo.js");
class CHTMLmo extends (0, mo_js_1.CommonMoMixin)(Wrapper_js_1.CHTMLWrapper) {
    toCHTML(parent) {
        const attributes = this.node.attributes;
        const symmetric = attributes.get('symmetric') && this.stretch.dir !== 2;
        const stretchy = this.stretch.dir !== 0;
        if (stretchy && this.size === null) {
            this.getStretchedVariant([]);
        }
        let chtml = this.standardCHTMLnode(parent);
        if (stretchy && this.size < 0) {
            this.stretchHTML(chtml);
        }
        else {
            if (symmetric || attributes.get('largeop')) {
                const u = this.em(this.getCenterOffset());
                if (u !== '0') {
                    this.adaptor.setStyle(chtml, 'verticalAlign', u);
                }
            }
            if (this.node.getProperty('mathaccent')) {
                this.adaptor.setStyle(chtml, 'width', '0');
                this.adaptor.setStyle(chtml, 'margin-left', this.em(this.getAccentOffset()));
            }
            for (const child of this.childNodes) {
                child.toCHTML(chtml);
            }
        }
    }
    stretchHTML(chtml) {
        const c = this.getText().codePointAt(0);
        this.font.delimUsage.add(c);
        this.childNodes[0].markUsed();
        const delim = this.stretch;
        const stretch = delim.stretch;
        const content = [];
        if (stretch[0]) {
            content.push(this.html('mjx-beg', {}, [this.html('mjx-c')]));
        }
        content.push(this.html('mjx-ext', {}, [this.html('mjx-c')]));
        if (stretch.length === 4) {
            content.push(this.html('mjx-mid', {}, [this.html('mjx-c')]), this.html('mjx-ext', {}, [this.html('mjx-c')]));
        }
        if (stretch[2]) {
            content.push(this.html('mjx-end', {}, [this.html('mjx-c')]));
        }
        const styles = {};
        const { h, d, w } = this.bbox;
        if (delim.dir === 1) {
            content.push(this.html('mjx-mark'));
            styles.height = this.em(h + d);
            styles.verticalAlign = this.em(-d);
        }
        else {
            styles.width = this.em(w);
        }
        const dir = mo_js_1.DirectionVH[delim.dir];
        const properties = { class: this.char(delim.c || c), style: styles };
        const html = this.html('mjx-stretchy-' + dir, properties, content);
        this.adaptor.append(chtml, html);
    }
}
exports.CHTMLmo = CHTMLmo;
CHTMLmo.kind = mo_js_2.MmlMo.prototype.kind;
CHTMLmo.styles = {
    'mjx-stretchy-h': {
        display: 'inline-table',
        width: '100%'
    },
    'mjx-stretchy-h > *': {
        display: 'table-cell',
        width: 0
    },
    'mjx-stretchy-h > * > mjx-c': {
        display: 'inline-block',
        transform: 'scalex(1.0000001)'
    },
    'mjx-stretchy-h > * > mjx-c::before': {
        display: 'inline-block',
        width: 'initial'
    },
    'mjx-stretchy-h > mjx-ext': {
        '/* IE */ overflow': 'hidden',
        '/* others */ overflow': 'clip visible',
        width: '100%'
    },
    'mjx-stretchy-h > mjx-ext > mjx-c::before': {
        transform: 'scalex(500)'
    },
    'mjx-stretchy-h > mjx-ext > mjx-c': {
        width: 0
    },
    'mjx-stretchy-h > mjx-beg > mjx-c': {
        'margin-right': '-.1em'
    },
    'mjx-stretchy-h > mjx-end > mjx-c': {
        'margin-left': '-.1em'
    },
    'mjx-stretchy-v': {
        display: 'inline-block'
    },
    'mjx-stretchy-v > *': {
        display: 'block'
    },
    'mjx-stretchy-v > mjx-beg': {
        height: 0
    },
    'mjx-stretchy-v > mjx-end > mjx-c': {
        display: 'block'
    },
    'mjx-stretchy-v > * > mjx-c': {
        transform: 'scaley(1.0000001)',
        'transform-origin': 'left center',
        overflow: 'hidden'
    },
    'mjx-stretchy-v > mjx-ext': {
        display: 'block',
        height: '100%',
        'box-sizing': 'border-box',
        border: '0px solid transparent',
        '/* IE */ overflow': 'hidden',
        '/* others */ overflow': 'visible clip',
    },
    'mjx-stretchy-v > mjx-ext > mjx-c::before': {
        width: 'initial',
        'box-sizing': 'border-box'
    },
    'mjx-stretchy-v > mjx-ext > mjx-c': {
        transform: 'scaleY(500) translateY(.075em)',
        overflow: 'visible'
    },
    'mjx-mark': {
        display: 'inline-block',
        height: '0px'
    }
};
//# sourceMappingURL=mo.js.map