"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonTextNodeMixin = void 0;
function CommonTextNodeMixin(Base) {
    return class extends Base {
        computeBBox(bbox, _recompute = false) {
            const variant = this.parent.variant;
            const text = this.node.getText();
            if (variant === '-explicitFont') {
                const font = this.jax.getFontData(this.parent.styles);
                const { w, h, d } = this.jax.measureText(text, variant, font);
                bbox.h = h;
                bbox.d = d;
                bbox.w = w;
            }
            else {
                const chars = this.remappedText(text, variant);
                bbox.empty();
                for (const char of chars) {
                    let [h, d, w, data] = this.getVariantChar(variant, char);
                    if (data.unknown) {
                        const cbox = this.jax.measureText(String.fromCodePoint(char), variant);
                        w = cbox.w;
                        h = cbox.h;
                        d = cbox.d;
                    }
                    bbox.w += w;
                    if (h > bbox.h)
                        bbox.h = h;
                    if (d > bbox.d)
                        bbox.d = d;
                    bbox.ic = data.ic || 0;
                    bbox.sk = data.sk || 0;
                    bbox.dx = data.dx || 0;
                }
                if (chars.length > 1) {
                    bbox.sk = 0;
                }
                bbox.clean();
            }
        }
        remappedText(text, variant) {
            const c = this.parent.stretch.c;
            return (c ? [c] : this.parent.remapChars(this.unicodeChars(text, variant)));
        }
        getStyles() { }
        getVariant() { }
        getScale() { }
        getSpace() { }
    };
}
exports.CommonTextNodeMixin = CommonTextNodeMixin;
//# sourceMappingURL=TextNode.js.map