"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonMactionMixin = exports.TooltipData = void 0;
const string_js_1 = require("../../../util/string.js");
exports.TooltipData = {
    dx: '.2em',
    dy: '.1em',
    postDelay: 600,
    clearDelay: 100,
    hoverTimer: new Map(),
    clearTimer: new Map(),
    stopTimers: (node, data) => {
        if (data.clearTimer.has(node)) {
            clearTimeout(data.clearTimer.get(node));
            data.clearTimer.delete(node);
        }
        if (data.hoverTimer.has(node)) {
            clearTimeout(data.hoverTimer.get(node));
            data.hoverTimer.delete(node);
        }
    }
};
function CommonMactionMixin(Base) {
    return class extends Base {
        get selected() {
            const selection = this.node.attributes.get('selection');
            const i = Math.max(1, Math.min(this.childNodes.length, selection)) - 1;
            return this.childNodes[i] || this.wrap(this.node.selected);
        }
        constructor(...args) {
            super(...args);
            const actions = this.constructor.actions;
            const action = this.node.attributes.get('actiontype');
            const [handler, data] = actions.get(action) || [((_node, _data) => { }), {}];
            this.action = handler;
            this.data = data;
            this.getParameters();
        }
        getParameters() {
            const offsets = this.node.attributes.get('data-offsets');
            let [dx, dy] = (0, string_js_1.split)(offsets || '');
            this.dx = this.length2em(dx || exports.TooltipData.dx);
            this.dy = this.length2em(dy || exports.TooltipData.dy);
        }
        computeBBox(bbox, recompute = false) {
            bbox.updateFrom(this.selected.getOuterBBox());
            this.selected.setChildPWidths(recompute);
        }
    };
}
exports.CommonMactionMixin = CommonMactionMixin;
//# sourceMappingURL=maction.js.map