"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonMoMixin = exports.DirectionVH = void 0;
const BBox_js_1 = require("../../../util/BBox.js");
const string_js_1 = require("../../../util/string.js");
const FontData_js_1 = require("../FontData.js");
exports.DirectionVH = {
    [1]: 'v',
    [2]: 'h'
};
function CommonMoMixin(Base) {
    return class extends Base {
        constructor(...args) {
            super(...args);
            this.size = null;
            this.isAccent = this.node.isAccent;
        }
        computeBBox(bbox, _recompute = false) {
            this.protoBBox(bbox);
            if (this.node.attributes.get('symmetric') &&
                this.stretch.dir !== 2) {
                const d = this.getCenterOffset(bbox);
                bbox.h += d;
                bbox.d -= d;
            }
            if (this.node.getProperty('mathaccent') &&
                (this.stretch.dir === 0 || this.size >= 0)) {
                bbox.w = 0;
            }
        }
        protoBBox(bbox) {
            const stretchy = (this.stretch.dir !== 0);
            if (stretchy && this.size === null) {
                this.getStretchedVariant([0]);
            }
            if (stretchy && this.size < 0)
                return;
            super.computeBBox(bbox);
            this.copySkewIC(bbox);
        }
        getAccentOffset() {
            const bbox = BBox_js_1.BBox.empty();
            this.protoBBox(bbox);
            return -bbox.w / 2;
        }
        getCenterOffset(bbox = null) {
            if (!bbox) {
                bbox = BBox_js_1.BBox.empty();
                super.computeBBox(bbox);
            }
            return ((bbox.h + bbox.d) / 2 + this.font.params.axis_height) - bbox.h;
        }
        getVariant() {
            if (this.node.attributes.get('largeop')) {
                this.variant = (this.node.attributes.get('displaystyle') ? '-largeop' : '-smallop');
                return;
            }
            if (!this.node.attributes.getExplicit('mathvariant') &&
                this.node.getProperty('pseudoscript') === false) {
                this.variant = '-tex-variant';
                return;
            }
            super.getVariant();
        }
        canStretch(direction) {
            if (this.stretch.dir !== 0) {
                return this.stretch.dir === direction;
            }
            const attributes = this.node.attributes;
            if (!attributes.get('stretchy'))
                return false;
            const c = this.getText();
            if (Array.from(c).length !== 1)
                return false;
            const delim = this.font.getDelimiter(c.codePointAt(0));
            this.stretch = (delim && delim.dir === direction ? delim : FontData_js_1.NOSTRETCH);
            return this.stretch.dir !== 0;
        }
        getStretchedVariant(WH, exact = false) {
            if (this.stretch.dir !== 0) {
                let D = this.getWH(WH);
                const min = this.getSize('minsize', 0);
                const max = this.getSize('maxsize', Infinity);
                const mathaccent = this.node.getProperty('mathaccent');
                D = Math.max(min, Math.min(max, D));
                const df = this.font.params.delimiterfactor / 1000;
                const ds = this.font.params.delimitershortfall;
                const m = (min || exact ? D : mathaccent ? Math.min(D / df, D + ds) : Math.max(D * df, D - ds));
                const delim = this.stretch;
                const c = delim.c || this.getText().codePointAt(0);
                let i = 0;
                if (delim.sizes) {
                    for (const d of delim.sizes) {
                        if (d >= m) {
                            if (mathaccent && i) {
                                i--;
                            }
                            this.variant = this.font.getSizeVariant(c, i);
                            this.size = i;
                            if (delim.schar && delim.schar[i]) {
                                this.stretch = Object.assign(Object.assign({}, this.stretch), { c: delim.schar[i] });
                            }
                            return;
                        }
                        i++;
                    }
                }
                if (delim.stretch) {
                    this.size = -1;
                    this.invalidateBBox();
                    this.getStretchBBox(WH, this.checkExtendedHeight(D, delim), delim);
                }
                else {
                    this.variant = this.font.getSizeVariant(c, i - 1);
                    this.size = i - 1;
                }
            }
        }
        getSize(name, value) {
            let attributes = this.node.attributes;
            if (attributes.isSet(name)) {
                value = this.length2em(attributes.get(name), 1, 1);
            }
            return value;
        }
        getWH(WH) {
            if (WH.length === 0)
                return 0;
            if (WH.length === 1)
                return WH[0];
            let [H, D] = WH;
            const a = this.font.params.axis_height;
            return (this.node.attributes.get('symmetric') ? 2 * Math.max(H - a, D + a) : H + D);
        }
        getStretchBBox(WHD, D, C) {
            if (C.hasOwnProperty('min') && C.min > D) {
                D = C.min;
            }
            let [h, d, w] = C.HDW;
            if (this.stretch.dir === 1) {
                [h, d] = this.getBaseline(WHD, D, C);
            }
            else {
                w = D;
            }
            this.bbox.h = h;
            this.bbox.d = d;
            this.bbox.w = w;
        }
        getBaseline(WHD, HD, C) {
            const hasWHD = (WHD.length === 2 && WHD[0] + WHD[1] === HD);
            const symmetric = this.node.attributes.get('symmetric');
            const [H, D] = (hasWHD ? WHD : [HD, 0]);
            let [h, d] = [H + D, 0];
            if (symmetric) {
                const a = this.font.params.axis_height;
                if (hasWHD) {
                    h = 2 * Math.max(H - a, D + a);
                }
                d = h / 2 - a;
            }
            else if (hasWHD) {
                d = D;
            }
            else {
                let [ch, cd] = (C.HDW || [.75, .25]);
                d = cd * (h / (ch + cd));
            }
            return [h - d, d];
        }
        checkExtendedHeight(D, C) {
            if (C.fullExt) {
                const [extSize, endSize] = C.fullExt;
                const n = Math.ceil(Math.max(0, D - endSize) / extSize);
                D = endSize + n * extSize;
            }
            return D;
        }
        remapChars(chars) {
            const primes = this.node.getProperty('primes');
            if (primes) {
                return (0, string_js_1.unicodeChars)(primes);
            }
            if (chars.length === 1) {
                const parent = this.node.coreParent().parent;
                const isAccent = this.isAccent && !parent.isKind('mrow');
                const map = (isAccent ? 'accent' : 'mo');
                const text = this.font.getRemappedChar(map, chars[0]);
                if (text) {
                    chars = this.unicodeChars(text, this.variant);
                }
            }
            return chars;
        }
    };
}
exports.CommonMoMixin = CommonMoMixin;
//# sourceMappingURL=mo.js.map