"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonMlabeledtrMixin = exports.CommonMtrMixin = void 0;
function CommonMtrMixin(Base) {
    return class extends Base {
        get fixesPWidth() {
            return false;
        }
        get numCells() {
            return this.childNodes.length;
        }
        get labeled() {
            return false;
        }
        get tableCells() {
            return this.childNodes;
        }
        getChild(i) {
            return this.childNodes[i];
        }
        getChildBBoxes() {
            return this.childNodes.map(cell => cell.getBBox());
        }
        stretchChildren(HD = null) {
            let stretchy = [];
            let children = (this.labeled ? this.childNodes.slice(1) : this.childNodes);
            for (const mtd of children) {
                const child = mtd.childNodes[0];
                if (child.canStretch(1)) {
                    stretchy.push(child);
                }
            }
            let count = stretchy.length;
            let nodeCount = this.childNodes.length;
            if (count && nodeCount > 1) {
                if (HD === null) {
                    let H = 0, D = 0;
                    let all = (count > 1 && count === nodeCount);
                    for (const mtd of children) {
                        const child = mtd.childNodes[0];
                        const noStretch = (child.stretch.dir === 0);
                        if (all || noStretch) {
                            const { h, d } = child.getBBox(noStretch);
                            if (h > H) {
                                H = h;
                            }
                            if (d > D) {
                                D = d;
                            }
                        }
                    }
                    HD = [H, D];
                }
                for (const child of stretchy) {
                    child.coreMO().getStretchedVariant(HD);
                }
            }
        }
    };
}
exports.CommonMtrMixin = CommonMtrMixin;
function CommonMlabeledtrMixin(Base) {
    return class extends Base {
        get numCells() {
            return Math.max(0, this.childNodes.length - 1);
        }
        get labeled() {
            return true;
        }
        get tableCells() {
            return this.childNodes.slice(1);
        }
        getChild(i) {
            return this.childNodes[i + 1];
        }
        getChildBBoxes() {
            return this.childNodes.slice(1).map(cell => cell.getBBox());
        }
    };
}
exports.CommonMlabeledtrMixin = CommonMlabeledtrMixin;
//# sourceMappingURL=mtr.js.map