"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVGmaction = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const maction_js_1 = require("../../common/Wrappers/maction.js");
const maction_js_2 = require("../../common/Wrappers/maction.js");
const maction_js_3 = require("../../../core/MmlTree/MmlNodes/maction.js");
class SVGmaction extends (0, maction_js_1.CommonMactionMixin)(Wrapper_js_1.SVGWrapper) {
    toSVG(parent) {
        const svg = this.standardSVGnode(parent);
        const child = this.selected;
        const { h, d, w } = child.getOuterBBox();
        this.adaptor.append(this.element, this.svg('rect', {
            width: this.fixed(w), height: this.fixed(h + d), y: this.fixed(-d),
            fill: 'none', 'pointer-events': 'all'
        }));
        child.toSVG(svg);
        const bbox = child.getOuterBBox();
        if (child.element) {
            child.place(bbox.L * bbox.rscale, 0);
        }
        this.action(this, this.data);
    }
    setEventHandler(type, handler) {
        this.element.addEventListener(type, handler);
    }
}
exports.SVGmaction = SVGmaction;
SVGmaction.kind = maction_js_3.MmlMaction.prototype.kind;
SVGmaction.styles = {
    '[jax="SVG"] mjx-tool': {
        display: 'inline-block',
        position: 'relative',
        width: 0, height: 0
    },
    '[jax="SVG"] mjx-tool > mjx-tip': {
        position: 'absolute',
        top: 0, left: 0
    },
    'mjx-tool > mjx-tip': {
        display: 'inline-block',
        padding: '.2em',
        border: '1px solid #888',
        'font-size': '70%',
        'background-color': '#F8F8F8',
        color: 'black',
        'box-shadow': '2px 2px 5px #AAAAAA'
    },
    'g[data-mml-node="maction"][data-toggle]': {
        cursor: 'pointer'
    },
    'mjx-status': {
        display: 'block',
        position: 'fixed',
        left: '1em',
        bottom: '1em',
        'min-width': '25%',
        padding: '.2em .4em',
        border: '1px solid #888',
        'font-size': '90%',
        'background-color': '#F8F8F8',
        color: 'black'
    }
};
SVGmaction.actions = new Map([
    ['toggle', [(node, _data) => {
                node.adaptor.setAttribute(node.element, 'data-toggle', node.node.attributes.get('selection'));
                const math = node.factory.jax.math;
                const document = node.factory.jax.document;
                const mml = node.node;
                node.setEventHandler('click', (event) => {
                    if (!math.end.node) {
                        math.start.node = math.end.node = math.typesetRoot;
                        math.start.n = math.end.n = 0;
                    }
                    mml.nextToggleSelection();
                    math.rerender(document);
                    event.stopPropagation();
                });
            }, {}]],
    ['tooltip', [(node, data) => {
                const tip = node.childNodes[1];
                if (!tip)
                    return;
                const rect = node.firstChild();
                if (tip.node.isKind('mtext')) {
                    const text = tip.node.getText();
                    node.adaptor.insert(node.svg('title', {}, [node.text(text)]), rect);
                }
                else {
                    const adaptor = node.adaptor;
                    const container = node.jax.container;
                    const math = node.node.factory.create('math', {}, [node.childNodes[1].node]);
                    const tool = node.html('mjx-tool', {}, [node.html('mjx-tip')]);
                    const hidden = adaptor.append(rect, node.svg('foreignObject', { style: { display: 'none' } }, [tool]));
                    node.jax.processMath(math, adaptor.firstChild(tool));
                    node.childNodes[1].node.parent = node.node;
                    node.setEventHandler('mouseover', (event) => {
                        data.stopTimers(node, data);
                        data.hoverTimer.set(node, setTimeout(() => {
                            adaptor.setStyle(tool, 'left', '0');
                            adaptor.setStyle(tool, 'top', '0');
                            adaptor.append(container, tool);
                            const tbox = adaptor.nodeBBox(tool);
                            const nbox = adaptor.nodeBBox(node.element);
                            const dx = (nbox.right - tbox.left) / node.metrics.em + node.dx;
                            const dy = (nbox.bottom - tbox.bottom) / node.metrics.em + node.dy;
                            adaptor.setStyle(tool, 'left', node.px(dx));
                            adaptor.setStyle(tool, 'top', node.px(dy));
                        }, data.postDelay));
                        event.stopPropagation();
                    });
                    node.setEventHandler('mouseout', (event) => {
                        data.stopTimers(node, data);
                        const timer = setTimeout(() => adaptor.append(hidden, tool), data.clearDelay);
                        data.clearTimer.set(node, timer);
                        event.stopPropagation();
                    });
                }
            }, maction_js_2.TooltipData]],
    ['statusline', [(node, data) => {
                const tip = node.childNodes[1];
                if (!tip)
                    return;
                if (tip.node.isKind('mtext')) {
                    const adaptor = node.adaptor;
                    const text = tip.node.getText();
                    adaptor.setAttribute(node.element, 'data-statusline', text);
                    node.setEventHandler('mouseover', (event) => {
                        if (data.status === null) {
                            const body = adaptor.body(adaptor.document);
                            data.status = adaptor.append(body, node.html('mjx-status', {}, [node.text(text)]));
                        }
                        event.stopPropagation();
                    });
                    node.setEventHandler('mouseout', (event) => {
                        if (data.status) {
                            adaptor.remove(data.status);
                            data.status = null;
                        }
                        event.stopPropagation();
                    });
                }
            }, {
                status: null
            }]]
]);
//# sourceMappingURL=maction.js.map