"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVGmath = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const math_js_1 = require("../../common/Wrappers/math.js");
const math_js_2 = require("../../../core/MmlTree/MmlNodes/math.js");
const BBox_js_1 = require("../../../util/BBox.js");
class SVGmath extends (0, math_js_1.CommonMathMixin)(Wrapper_js_1.SVGWrapper) {
    toSVG(parent) {
        super.toSVG(parent);
        const adaptor = this.adaptor;
        const display = (this.node.attributes.get('display') === 'block');
        if (display) {
            adaptor.setAttribute(this.jax.container, 'display', 'true');
            this.handleDisplay();
        }
        if (this.jax.document.options.internalSpeechTitles) {
            this.handleSpeech();
        }
    }
    handleDisplay() {
        const [align, shift] = this.getAlignShift();
        if (align !== 'center') {
            this.adaptor.setAttribute(this.jax.container, 'justify', align);
        }
        if (this.bbox.pwidth === BBox_js_1.BBox.fullWidth) {
            this.adaptor.setAttribute(this.jax.container, 'width', 'full');
            if (this.jax.table) {
                let { L, w, R } = this.jax.table.getOuterBBox();
                if (align === 'right') {
                    R = Math.max(R || -shift, -shift);
                }
                else if (align === 'left') {
                    L = Math.max(L || shift, shift);
                }
                else if (align === 'center') {
                    w += 2 * Math.abs(shift);
                }
                this.jax.minwidth = Math.max(0, L + w + R);
            }
        }
        else {
            this.jax.shift = shift;
        }
    }
    handleSpeech() {
        const adaptor = this.adaptor;
        const attributes = this.node.attributes;
        const speech = (attributes.get('aria-label') || attributes.get('data-semantic-speech'));
        if (speech) {
            const id = this.getTitleID();
            const label = this.svg('title', { id }, [this.text(speech)]);
            adaptor.insert(label, adaptor.firstChild(this.element));
            adaptor.setAttribute(this.element, 'aria-labeledby', id);
            adaptor.removeAttribute(this.element, 'aria-label');
            for (const child of this.childNodes[0].childNodes) {
                adaptor.setAttribute(child.element, 'aria-hidden', 'true');
            }
        }
    }
    getTitleID() {
        return 'mjx-svg-title-' + String(this.jax.options.titleID++);
    }
    setChildPWidths(recompute, w = null, _clear = true) {
        return super.setChildPWidths(recompute, this.parent ? w : this.metrics.containerWidth / this.jax.pxPerEm, false);
    }
}
exports.SVGmath = SVGmath;
SVGmath.kind = math_js_2.MmlMath.prototype.kind;
SVGmath.styles = {
    'mjx-container[jax="SVG"][display="true"]': {
        display: 'block',
        'text-align': 'center',
        margin: '1em 0'
    },
    'mjx-container[jax="SVG"][display="true"][width="full"]': {
        display: 'flex'
    },
    'mjx-container[jax="SVG"][justify="left"]': {
        'text-align': 'left'
    },
    'mjx-container[jax="SVG"][justify="right"]': {
        'text-align': 'right'
    }
};
//# sourceMappingURL=math.js.map