"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVGmfrac = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const mfrac_js_1 = require("../../common/Wrappers/mfrac.js");
const mfrac_js_2 = require("../../../core/MmlTree/MmlNodes/mfrac.js");
class SVGmfrac extends (0, mfrac_js_1.CommonMfracMixin)(Wrapper_js_1.SVGWrapper) {
    toSVG(parent) {
        this.standardSVGnode(parent);
        const { linethickness, bevelled } = this.node.attributes.getList('linethickness', 'bevelled');
        const display = this.isDisplay();
        if (bevelled) {
            this.makeBevelled(display);
        }
        else {
            const thickness = this.length2em(String(linethickness), .06);
            if (thickness === 0) {
                this.makeAtop(display);
            }
            else {
                this.makeFraction(display, thickness);
            }
        }
    }
    makeFraction(display, t) {
        const svg = this.element;
        const { numalign, denomalign } = this.node.attributes.getList('numalign', 'denomalign');
        const [num, den] = this.childNodes;
        const nbox = num.getOuterBBox();
        const dbox = den.getOuterBBox();
        const tex = this.font.params;
        const a = tex.axis_height;
        const d = .1;
        const pad = (this.node.getProperty('withDelims') ? 0 : tex.nulldelimiterspace);
        const W = Math.max((nbox.L + nbox.w + nbox.R) * nbox.rscale, (dbox.L + dbox.w + dbox.R) * dbox.rscale);
        const nx = this.getAlignX(W, nbox, numalign) + d + pad;
        const dx = this.getAlignX(W, dbox, denomalign) + d + pad;
        const { T, u, v } = this.getTUV(display, t);
        num.toSVG(svg);
        num.place(nx, a + T + Math.max(nbox.d * nbox.rscale, u));
        den.toSVG(svg);
        den.place(dx, a - T - Math.max(dbox.h * dbox.rscale, v));
        this.adaptor.append(svg, this.svg('rect', {
            width: this.fixed(W + 2 * d), height: this.fixed(t),
            x: this.fixed(pad), y: this.fixed(a - t / 2)
        }));
    }
    makeAtop(display) {
        const svg = this.element;
        const { numalign, denomalign } = this.node.attributes.getList('numalign', 'denomalign');
        const [num, den] = this.childNodes;
        const nbox = num.getOuterBBox();
        const dbox = den.getOuterBBox();
        const tex = this.font.params;
        const pad = (this.node.getProperty('withDelims') ? 0 : tex.nulldelimiterspace);
        const W = Math.max((nbox.L + nbox.w + nbox.R) * nbox.rscale, (dbox.L + dbox.w + dbox.R) * dbox.rscale);
        const nx = this.getAlignX(W, nbox, numalign) + pad;
        const dx = this.getAlignX(W, dbox, denomalign) + pad;
        const { u, v } = this.getUVQ(display);
        num.toSVG(svg);
        num.place(nx, u);
        den.toSVG(svg);
        den.place(dx, -v);
    }
    makeBevelled(display) {
        const svg = this.element;
        const [num, den] = this.childNodes;
        const { u, v, delta, nbox, dbox } = this.getBevelData(display);
        const w = (nbox.L + nbox.w + nbox.R) * nbox.rscale;
        num.toSVG(svg);
        this.bevel.toSVG(svg);
        den.toSVG(svg);
        num.place(nbox.L * nbox.rscale, u);
        this.bevel.place(w - delta / 2, 0);
        den.place(w + this.bevel.getOuterBBox().w + dbox.L * dbox.rscale - delta, v);
    }
}
exports.SVGmfrac = SVGmfrac;
SVGmfrac.kind = mfrac_js_2.MmlMfrac.prototype.kind;
//# sourceMappingURL=mfrac.js.map