"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVGmlabeledtr = exports.SVGmtr = void 0;
const Wrapper_js_1 = require("../Wrapper.js");
const mtr_js_1 = require("../../common/Wrappers/mtr.js");
const mtr_js_2 = require("../../common/Wrappers/mtr.js");
const mtr_js_3 = require("../../../core/MmlTree/MmlNodes/mtr.js");
class SVGmtr extends (0, mtr_js_1.CommonMtrMixin)(Wrapper_js_1.SVGWrapper) {
    toSVG(parent) {
        const svg = this.standardSVGnode(parent);
        this.placeCells(svg);
        this.placeColor();
    }
    placeCells(svg) {
        const cSpace = this.parent.getColumnHalfSpacing();
        const cLines = [this.parent.fLine, ...this.parent.cLines, this.parent.fLine];
        const cWidth = this.parent.getComputedWidths();
        const scale = 1 / this.getBBox().rscale;
        let x = cLines[0];
        for (let i = 0; i < this.numCells; i++) {
            const child = this.getChild(i);
            child.toSVG(svg);
            x += this.placeCell(child, {
                x: x, y: 0, lSpace: cSpace[i] * scale, rSpace: cSpace[i + 1] * scale, w: cWidth[i] * scale,
                lLine: cLines[i] * scale, rLine: cLines[i + 1] * scale
            });
        }
    }
    placeCell(cell, sizes) {
        const { x, y, lSpace, w, rSpace, lLine, rLine } = sizes;
        const scale = 1 / this.getBBox().rscale;
        const [h, d] = [this.H * scale, this.D * scale];
        const [t, b] = [this.tSpace * scale, this.bSpace * scale];
        const [dx, dy] = cell.placeCell(x + lSpace, y, w, h, d);
        const W = lSpace + w + rSpace;
        cell.placeColor(-(dx + lSpace + lLine / 2), -(d + b + dy), W + (lLine + rLine) / 2, h + d + t + b);
        return W + rLine;
    }
    placeColor() {
        const scale = 1 / this.getBBox().rscale;
        const adaptor = this.adaptor;
        const child = this.firstChild();
        if (child && adaptor.kind(child) === 'rect' && adaptor.getAttribute(child, 'data-bgcolor')) {
            const [TL, BL] = [(this.tLine / 2) * scale, (this.bLine / 2) * scale];
            const [TS, BS] = [this.tSpace * scale, this.bSpace * scale];
            const [H, D] = [this.H * scale, this.D * scale];
            adaptor.setAttribute(child, 'y', this.fixed(-(D + BS + BL)));
            adaptor.setAttribute(child, 'width', this.fixed(this.parent.getWidth() * scale));
            adaptor.setAttribute(child, 'height', this.fixed(TL + TS + H + D + BS + BL));
        }
    }
}
exports.SVGmtr = SVGmtr;
SVGmtr.kind = mtr_js_3.MmlMtr.prototype.kind;
class SVGmlabeledtr extends (0, mtr_js_2.CommonMlabeledtrMixin)(SVGmtr) {
    toSVG(parent) {
        super.toSVG(parent);
        const child = this.childNodes[0];
        if (child) {
            child.toSVG(this.parent.labels);
        }
    }
}
exports.SVGmlabeledtr = SVGmlabeledtr;
SVGmlabeledtr.kind = mtr_js_3.MmlMlabeledtr.prototype.kind;
//# sourceMappingURL=mtr.js.map