"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeXFont = void 0;
const FontData_js_1 = require("../FontData.js");
const tex_js_1 = require("../../common/fonts/tex.js");
const bold_italic_js_1 = require("./tex/bold-italic.js");
const bold_js_1 = require("./tex/bold.js");
const double_struck_js_1 = require("./tex/double-struck.js");
const fraktur_bold_js_1 = require("./tex/fraktur-bold.js");
const fraktur_js_1 = require("./tex/fraktur.js");
const italic_js_1 = require("./tex/italic.js");
const largeop_js_1 = require("./tex/largeop.js");
const monospace_js_1 = require("./tex/monospace.js");
const normal_js_1 = require("./tex/normal.js");
const sans_serif_bold_italic_js_1 = require("./tex/sans-serif-bold-italic.js");
const sans_serif_bold_js_1 = require("./tex/sans-serif-bold.js");
const sans_serif_italic_js_1 = require("./tex/sans-serif-italic.js");
const sans_serif_js_1 = require("./tex/sans-serif.js");
const script_bold_js_1 = require("./tex/script-bold.js");
const script_js_1 = require("./tex/script.js");
const smallop_js_1 = require("./tex/smallop.js");
const tex_calligraphic_bold_js_1 = require("./tex/tex-calligraphic-bold.js");
const tex_calligraphic_js_1 = require("./tex/tex-calligraphic.js");
const tex_mathit_js_1 = require("./tex/tex-mathit.js");
const tex_oldstyle_bold_js_1 = require("./tex/tex-oldstyle-bold.js");
const tex_oldstyle_js_1 = require("./tex/tex-oldstyle.js");
const tex_size3_js_1 = require("./tex/tex-size3.js");
const tex_size4_js_1 = require("./tex/tex-size4.js");
const tex_variant_js_1 = require("./tex/tex-variant.js");
const delimiters_js_1 = require("../../common/fonts/tex/delimiters.js");
class TeXFont extends (0, tex_js_1.CommonTeXFontMixin)(FontData_js_1.SVGFontData) {
    constructor(options = null) {
        super(options);
        const CLASS = this.constructor;
        for (const variant of Object.keys(CLASS.variantCacheIds)) {
            this.variant[variant].cacheID = 'TEX-' + CLASS.variantCacheIds[variant];
        }
    }
}
exports.TeXFont = TeXFont;
TeXFont.defaultDelimiters = delimiters_js_1.delimiters;
TeXFont.defaultChars = {
    'normal': normal_js_1.normal,
    'bold': bold_js_1.bold,
    'italic': italic_js_1.italic,
    'bold-italic': bold_italic_js_1.boldItalic,
    'double-struck': double_struck_js_1.doubleStruck,
    'fraktur': fraktur_js_1.fraktur,
    'bold-fraktur': fraktur_bold_js_1.frakturBold,
    'script': script_js_1.script,
    'bold-script': script_bold_js_1.scriptBold,
    'sans-serif': sans_serif_js_1.sansSerif,
    'bold-sans-serif': sans_serif_bold_js_1.sansSerifBold,
    'sans-serif-italic': sans_serif_italic_js_1.sansSerifItalic,
    'sans-serif-bold-italic': sans_serif_bold_italic_js_1.sansSerifBoldItalic,
    'monospace': monospace_js_1.monospace,
    '-smallop': smallop_js_1.smallop,
    '-largeop': largeop_js_1.largeop,
    '-size3': tex_size3_js_1.texSize3,
    '-size4': tex_size4_js_1.texSize4,
    '-tex-calligraphic': tex_calligraphic_js_1.texCalligraphic,
    '-tex-bold-calligraphic': tex_calligraphic_bold_js_1.texCalligraphicBold,
    '-tex-mathit': tex_mathit_js_1.texMathit,
    '-tex-oldstyle': tex_oldstyle_js_1.texOldstyle,
    '-tex-bold-oldstyle': tex_oldstyle_bold_js_1.texOldstyleBold,
    '-tex-variant': tex_variant_js_1.texVariant
};
TeXFont.variantCacheIds = {
    'normal': 'N',
    'bold': 'B',
    'italic': 'I',
    'bold-italic': 'BI',
    'double-struck': 'D',
    'fraktur': 'F',
    'bold-fraktur': 'BF',
    'script': 'S',
    'bold-script': 'BS',
    'sans-serif': 'SS',
    'bold-sans-serif': 'BSS',
    'sans-serif-italic': 'SSI',
    'sans-serif-bold-italic': 'SSBI',
    'monospace': 'M',
    '-smallop': 'SO',
    '-largeop': 'LO',
    '-size3': 'S3',
    '-size4': 'S4',
    '-tex-calligraphic': 'C',
    '-tex-bold-calligraphic': 'BC',
    '-tex-mathit': 'MI',
    '-tex-oldstyle': 'OS',
    '-tex-bold-oldstyle': 'BOS',
    '-tex-variant': 'V'
};
//# sourceMappingURL=tex.js.map