"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MJContextMenu = void 0;
const context_menu_js_1 = require("mj-context-menu/js/context_menu.js");
const item_submenu_js_1 = require("mj-context-menu/js/item_submenu.js");
class MJContextMenu extends context_menu_js_1.ContextMenu {
    constructor() {
        super(...arguments);
        this.mathItem = null;
        this.annotation = '';
        this.annotationTypes = {};
    }
    post(x, y) {
        if (this.mathItem) {
            if (y !== undefined) {
                const input = this.mathItem.inputJax.name;
                const original = this.findID('Show', 'Original');
                original.content = (input === 'MathML' ? 'Original MathML' : input + ' Commands');
                const clipboard = this.findID('Copy', 'Original');
                clipboard.content = original.content;
                const semantics = this.findID('Settings', 'semantics');
                input === 'MathML' ? semantics.disable() : semantics.enable();
                this.getAnnotationMenu();
                this.dynamicSubmenus();
            }
            super.post(x, y);
        }
    }
    unpost() {
        super.unpost();
        this.mathItem = null;
    }
    findID(...names) {
        let menu = this;
        let item = null;
        for (const name of names) {
            if (menu) {
                item = menu.find(name);
                menu = (item instanceof item_submenu_js_1.Submenu ? item.submenu : null);
            }
            else {
                item = null;
            }
        }
        return item;
    }
    getAnnotationMenu() {
        const annotations = this.getAnnotations(this.getSemanticNode());
        this.createAnnotationMenu('Show', annotations, () => this.showAnnotation.post());
        this.createAnnotationMenu('Copy', annotations, () => this.copyAnnotation());
    }
    getSemanticNode() {
        let node = this.mathItem.root;
        while (node && !node.isKind('semantics')) {
            if (node.isToken || node.childNodes.length !== 1)
                return null;
            node = node.childNodes[0];
        }
        return node;
    }
    getAnnotations(node) {
        const annotations = [];
        if (!node)
            return annotations;
        for (const child of node.childNodes) {
            if (child.isKind('annotation')) {
                const match = this.annotationMatch(child);
                if (match) {
                    const value = child.childNodes.reduce((text, chars) => text + chars.toString(), '');
                    annotations.push([match, value]);
                }
            }
        }
        return annotations;
    }
    annotationMatch(child) {
        const encoding = child.attributes.get('encoding');
        for (const type of Object.keys(this.annotationTypes)) {
            if (this.annotationTypes[type].indexOf(encoding) >= 0) {
                return type;
            }
        }
        return null;
    }
    createAnnotationMenu(id, annotations, action) {
        const menu = this.findID(id, 'Annotation');
        menu.submenu = this.factory.get('subMenu')(this.factory, {
            items: annotations.map(([type, value]) => {
                return {
                    type: 'command',
                    id: type,
                    content: type,
                    action: () => {
                        this.annotation = value;
                        action();
                    }
                };
            }),
            id: 'annotations'
        }, menu);
        if (annotations.length) {
            menu.enable();
        }
        else {
            menu.disable();
        }
    }
    dynamicSubmenus() {
        for (const [id, method] of MJContextMenu.DynamicSubmenus) {
            const menu = this.find(id);
            if (!menu)
                continue;
            const sub = method(this, menu);
            menu.submenu = sub;
            if (sub.items.length) {
                menu.enable();
            }
            else {
                menu.disable();
            }
        }
    }
}
exports.MJContextMenu = MJContextMenu;
MJContextMenu.DynamicSubmenus = new Map();
//# sourceMappingURL=MJContextMenu.js.map