"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuHandler = exports.MenuMathDocumentMixin = exports.MenuMathItemMixin = void 0;
const mathjax_js_1 = require("../../mathjax.js");
const MathItem_js_1 = require("../../core/MathItem.js");
const Options_js_1 = require("../../util/Options.js");
const Menu_js_1 = require("./Menu.js");
(0, MathItem_js_1.newState)('CONTEXT_MENU', 170);
function MenuMathItemMixin(BaseMathItem) {
    return class extends BaseMathItem {
        addMenu(document, force = false) {
            if (this.state() >= MathItem_js_1.STATE.CONTEXT_MENU)
                return;
            if (!this.isEscaped && (document.options.enableMenu || force)) {
                document.menu.addMenu(this);
            }
            this.state(MathItem_js_1.STATE.CONTEXT_MENU);
        }
        checkLoading(document) {
            document.checkLoading();
        }
    };
}
exports.MenuMathItemMixin = MenuMathItemMixin;
function MenuMathDocumentMixin(BaseDocument) {
    var _a;
    return _a = class extends BaseDocument {
            constructor(...args) {
                super(...args);
                this.menu = new this.options.MenuClass(this, this.options.menuOptions);
                const ProcessBits = this.constructor.ProcessBits;
                if (!ProcessBits.has('context-menu')) {
                    ProcessBits.allocate('context-menu');
                }
                this.options.MathItem = MenuMathItemMixin(this.options.MathItem);
            }
            addMenu() {
                if (!this.processed.isSet('context-menu')) {
                    for (const math of this.math) {
                        math.addMenu(this);
                    }
                    this.processed.set('context-menu');
                }
                return this;
            }
            checkLoading() {
                if (this.menu.isLoading) {
                    mathjax_js_1.mathjax.retryAfter(this.menu.loadingPromise.catch((err) => console.log(err)));
                }
                const settings = this.menu.settings;
                if (settings.collapsible) {
                    this.options.enableComplexity = true;
                    this.menu.checkComponent('a11y/complexity');
                }
                if (settings.explorer) {
                    this.options.enableEnrichment = true;
                    this.options.enableExplorer = true;
                    this.menu.checkComponent('a11y/explorer');
                }
                return this;
            }
            state(state, restore = false) {
                super.state(state, restore);
                if (state < MathItem_js_1.STATE.CONTEXT_MENU) {
                    this.processed.clear('context-menu');
                }
                return this;
            }
            updateDocument() {
                super.updateDocument();
                this.menu.menu.store.sort();
                return this;
            }
        },
        _a.OPTIONS = Object.assign(Object.assign({ enableEnrichment: true, enableComplexity: true, enableExplorer: true, enrichSpeech: 'none', enrichError: (_doc, _math, err) => console.warn('Enrichment Error:', err) }, BaseDocument.OPTIONS), { MenuClass: Menu_js_1.Menu, menuOptions: Menu_js_1.Menu.OPTIONS, enableMenu: true, sre: (BaseDocument.OPTIONS.sre || (0, Options_js_1.expandable)({})), a11y: (BaseDocument.OPTIONS.a11y || (0, Options_js_1.expandable)({})), renderActions: (0, Options_js_1.expandable)(Object.assign(Object.assign({}, BaseDocument.OPTIONS.renderActions), { addMenu: [MathItem_js_1.STATE.CONTEXT_MENU], checkLoading: [MathItem_js_1.STATE.UNPROCESSED + 1] })) }),
        _a;
}
exports.MenuMathDocumentMixin = MenuMathDocumentMixin;
function MenuHandler(handler) {
    handler.documentClass = MenuMathDocumentMixin(handler.documentClass);
    return handler;
}
exports.MenuHandler = MenuHandler;
//# sourceMappingURL=MenuHandler.js.map