"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectableInfo = void 0;
const info_js_1 = require("mj-context-menu/js/info.js");
const html_classes_js_1 = require("mj-context-menu/js/html_classes.js");
class SelectableInfo extends info_js_1.Info {
    addEvents(element) {
        element.addEventListener('keypress', (event) => {
            if (event.key === 'a' && (event.ctrlKey || event.metaKey)) {
                this.selectAll();
                this.stop(event);
            }
        });
    }
    selectAll() {
        const selection = document.getSelection();
        selection.selectAllChildren(this.html.querySelector('pre'));
    }
    copyToClipboard() {
        this.selectAll();
        try {
            document.execCommand('copy');
        }
        catch (err) {
            alert('Can\'t copy to clipboard: ' + err.message);
        }
        document.getSelection().removeAllRanges();
    }
    generateHtml() {
        super.generateHtml();
        const footer = this.html.querySelector('span.' + html_classes_js_1.HtmlClasses['INFOSIGNATURE']);
        const button = footer.appendChild(document.createElement('input'));
        button.type = 'button';
        button.value = 'Copy to Clipboard';
        button.addEventListener('click', (_event) => this.copyToClipboard());
    }
}
exports.SelectableInfo = SelectableInfo;
//# sourceMappingURL=SelectableInfo.js.map