"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BBox = void 0;
const lengths_js_1 = require("./lengths.js");
class BBox {
    static zero() {
        return new BBox({ h: 0, d: 0, w: 0 });
    }
    static empty() {
        return new BBox();
    }
    constructor(def = { w: 0, h: -lengths_js_1.BIGDIMEN, d: -lengths_js_1.BIGDIMEN }) {
        this.w = def.w || 0;
        this.h = ('h' in def ? def.h : -lengths_js_1.BIGDIMEN);
        this.d = ('d' in def ? def.d : -lengths_js_1.BIGDIMEN);
        this.L = this.R = this.ic = this.sk = this.dx = 0;
        this.scale = this.rscale = 1;
        this.pwidth = '';
    }
    empty() {
        this.w = 0;
        this.h = this.d = -lengths_js_1.BIGDIMEN;
        return this;
    }
    clean() {
        if (this.w === -lengths_js_1.BIGDIMEN)
            this.w = 0;
        if (this.h === -lengths_js_1.BIGDIMEN)
            this.h = 0;
        if (this.d === -lengths_js_1.BIGDIMEN)
            this.d = 0;
    }
    rescale(scale) {
        this.w *= scale;
        this.h *= scale;
        this.d *= scale;
    }
    combine(cbox, x = 0, y = 0) {
        let rscale = cbox.rscale;
        let w = x + rscale * (cbox.w + cbox.L + cbox.R);
        let h = y + rscale * cbox.h;
        let d = rscale * cbox.d - y;
        if (w > this.w)
            this.w = w;
        if (h > this.h)
            this.h = h;
        if (d > this.d)
            this.d = d;
    }
    append(cbox) {
        let scale = cbox.rscale;
        this.w += scale * (cbox.w + cbox.L + cbox.R);
        if (scale * cbox.h > this.h) {
            this.h = scale * cbox.h;
        }
        if (scale * cbox.d > this.d) {
            this.d = scale * cbox.d;
        }
    }
    updateFrom(cbox) {
        this.h = cbox.h;
        this.d = cbox.d;
        this.w = cbox.w;
        if (cbox.pwidth) {
            this.pwidth = cbox.pwidth;
        }
    }
}
exports.BBox = BBox;
BBox.fullWidth = '100%';
BBox.StyleAdjust = [
    ['borderTopWidth', 'h'],
    ['borderRightWidth', 'w'],
    ['borderBottomWidth', 'd'],
    ['borderLeftWidth', 'w', 0],
    ['paddingTop', 'h'],
    ['paddingRight', 'w'],
    ['paddingBottom', 'd'],
    ['paddingLeft', 'w', 0]
];
//# sourceMappingURL=BBox.js.map